// CCar Class Header
// 
// Description : base car class, handles everything car-like
//

#pragma once

// includes
#include "GU_v2.2.h"
#include "3DModel.h"

// defines (used in collision detection)
#define NONE	0
#define LEFT	1
#define RIGHT	2
#define BACK	3
#define FRONT	4


///////////////////////////////////////////////////////////////////
// CCar Class
// Description:	base class for all "car-like" functions
///////////////////////////////////////////////////////////////////
class CCar
{
public:

					CCar();
	virtual			~CCar()							{}
					CCar(CCar* car)					{	// copy constructor
														m_cCarModel = car->m_cCarModel;
														m_ptPosition = car->m_ptPosition;
														m_ptVelocityVector = car->m_ptVelocityVector;
														m_ptDirection = car->m_ptDirection;
														m_fSpeed = car->m_fSpeed;
														m_fRotation = car->m_fRotation;	
													}

			// init member variables at start of race
			void	InitializeParameters();

			// generic update
	virtual void	Update();

			// load, draw car
	virtual void	Draw(void);
			bool	Load(string strModelName)		{ return(m_cCarModel.Load(strModelName)); }

			// tell car to speed up, slow down, or coast
			void	SpeedUp();
			void	Brake();
			void	Coast();

			// tell car to turn
			void	TurnLeft(void);
			void	TurnRight(void);

			// tell car to finally move (after speeding up, etc)
			bool	Move(int* pEnvironment);
			int		CheckColliding(CCar* pCar);
			int		CheckCollidingPt(CPnt ptCar);

			// implicitly change speed of car
			void	SetSpeed(float fSpeed)			{ m_fSpeed = fSpeed; }
			float	GetSpeed(void) const			{ return m_fSpeed; }
			void	AddSpeed(float fIncrement)		{ m_fSpeed += fIncrement; }
			
			// get and set position, velocity, direction, and rotation
			CPnt	GetPos(void) const				{ return m_ptPosition; }
			CPnt	GetVel(void) const				{ return m_ptVelocityVector; }
			CPnt	GetDir(void) const				{ return m_ptDirection; }
			float	GetRot(void) const				{ return m_fRotation; }
	virtual	void	SetPos(CPnt pntPos)				{ m_ptPosition = pntPos; }
			void	SetVel(CPnt pntVel)				{ m_ptVelocityVector = pntVel; }
			void	SetDir(CPnt pntDir)				{ m_ptDirection = pntDir; }
			void	SetRot(float fRot)				{ m_fRotation = fRot; }

			// get and set place of car on track
			void	SetPlace(int nPlace)			{ m_nPlace = nPlace; }
			int		GetPlace() const				{ return m_nPlace; }
			string	GetPlaceString();				// format place as string for drawing

			// get lap
			int		GetLap() const					{ return m_nLap; }
			string	GetLapString();					// format lap as a string for drawing
			int		GetQuadrant() const				{ return m_nQuadrant; }	

			// set and get max number of laps in race
			void	SetNumLaps(int laps)			{ m_nNumLaps = laps; }
			string	GetNumLapsString();				// format as string for drawing

			// get and set start time of car
			void	SetStartTime(float time)		{ m_fStartTime = time; }
			float	GetStartTime()	const			{ return m_fStartTime; }

			// set and get whether car is finished race or not
			void	SetFinished(bool state)			{ m_bFinished = state; }
			bool	GetFinished() const				{ return m_bFinished; }			
			bool	WonRace() const					{ return m_bWonRace; }
			bool	SetWonRace(bool wonRace)		{ m_bWonRace = wonRace; }
				
// member variables
private:

	C3DModel		m_cCarModel;

	CPnt			m_ptPosition;
	CPnt			m_ptVelocityVector;
	CPnt			m_ptDirection;

	float			m_fSpeed;
	float			m_fRotation;
	float			m_fStartTime;

	bool			m_bLap;
	bool			m_bFinished;
	bool			m_bWonRace;

	void			SetQuadrant();

	int				m_nQuadrant;
	int				m_nLap;
	int				m_nNumLaps;
	int				m_nPlace;

public:
	// car's min and max speed
	float			m_fMaxSpeed;
	float			m_fMinSpeed;

	// display list index for text drawing
	GLuint			m_dlText;

	bool			m_bIsTurning;
};