// CEnvironment Class Implementation
// 
// Description : container class for all objects in game world
//

// includes
#include "stdafx.h"
#include "Environment.h"

///////////////////////////////////////////////////////////////////
// CEnvironment::InitializeEnvironment()
// Description:	inits parameters to reset game
///////////////////////////////////////////////////////////////////
void CEnvironment::InitializeEnvironment()
{
	m_acEnemyCars[0].InitializeParameters();
	m_acEnemyCars[1].InitializeParameters();
	m_acEnemyCars[2].InitializeParameters();
	m_acEnemyCars[3].InitializeParameters();
	m_acEnemyCars[4].InitializeParameters();
	m_acEnemyCars[5].InitializeParameters();
	m_acEnemyCars[6].InitializeParameters();
	m_acEnemyCars[7].InitializeParameters();
	m_cUserCar.InitializeParameters();
	
	m_acEnemyCars[0].SetPos(CPnt(-40.0f, 290.0f, 0.0f));
	m_acEnemyCars[1].SetPos(CPnt(-40.0f, 270.0f, 0.0f));
	m_acEnemyCars[2].SetPos(CPnt(-20.0f, 310.0f, 0.0f));
	m_acEnemyCars[3].SetPos(CPnt(-20.0f, 290.0f, 0.0f));
	m_acEnemyCars[4].SetPos(CPnt(-20.0f, 270.0f, 0.0f));
	m_acEnemyCars[5].SetPos(CPnt(-0.0f, 310.0f, 0.0f));
	m_acEnemyCars[6].SetPos(CPnt(-0.0f, 290.0f, 0.0f));
	m_acEnemyCars[7].SetPos(CPnt(-40.0f, 310.0f, 0.0f));
	m_cUserCar.SetPos(CPnt(0.0f, 270.0f, 0.0f));
	
	// set and distribute the start time
	m_bJustCrossedFinishLine = true;	
	SetStartTime();
	m_cUserCar.SetStartTime(GetStartTime());
	SetNumLaps(5);
	m_cUserCar.SetNumLaps(GetNumLaps());
	m_fStartZ = 50.0f;
	m_fRotateCamera = 360.0f;

	m_cUserCar.InitCamera();
	m_cUserCar.StartIntroTimer();

}

///////////////////////////////////////////////////////////////////
// InitializeHighScoreScreen()
// Description:	inits parameters for high score screen shot
///////////////////////////////////////////////////////////////////
void CEnvironment::InitializeHighScoreScreen()
{
	m_acEnemyCars[0].InitializeParameters();
	m_acEnemyCars[1].InitializeParameters();
	m_acEnemyCars[2].InitializeParameters();
	m_acEnemyCars[3].InitializeParameters();
	m_acEnemyCars[4].InitializeParameters();
	m_acEnemyCars[5].InitializeParameters();
	m_acEnemyCars[6].InitializeParameters();
	m_acEnemyCars[7].InitializeParameters();
	m_cUserCar.InitializeParameters();

	m_acEnemyCars[0].SetPos(CPnt(-40.0f, 285.0f, 0.0f));
	m_acEnemyCars[1].SetPos(CPnt(-40.0f, 270.0f, 0.0f));
	m_acEnemyCars[2].SetPos(CPnt(-20.0f, 300.0f, 0.0f));
	m_acEnemyCars[3].SetPos(CPnt(-20.0f, 285.0f, 0.0f));
	m_acEnemyCars[4].SetPos(CPnt(-20.0f, 270.0f, 0.0f));
	m_acEnemyCars[5].SetPos(CPnt(-0.0f, 300.0f, 0.0f));
	m_acEnemyCars[6].SetPos(CPnt(-0.0f, 285.0f, 0.0f));
	m_acEnemyCars[7].SetPos(CPnt(-40.0f, 300.0f, 0.0f));

	UpdateHighScoreCamera();
	m_cUserCar.InitCamera();
}


///////////////////////////////////////////////////////////////////
// CEnvironment::Update(bool bUp, bool bDown, bool bLeft, bool bRight, UINT chKey)
// Description:	updates game world, passes key messages to user car
///////////////////////////////////////////////////////////////////
bool CEnvironment::Update(bool bUp, bool bDown, bool bLeft, bool bRight, UINT chKey)
{
	int i;
	if(chKey == VK_TAB)
		m_cUserCar.ToggleViewMode();

	CalcCarPlace(&m_cUserCar);
	bool bCrash = m_cUserCar.Update(bUp, bDown, bLeft, bRight, (int*)this);

	for(i = 0; i < NUM_ENEMY_CARS; i++)
	{
		CalcCarPlace(&(m_acEnemyCars[i]));
		m_acEnemyCars[i].Update((int*)this);
	}

	//	Determine if winner
	if(GetNumLaps()+1 == m_cUserCar.GetLap() && m_bJustCrossedFinishLine)
	{
		m_cUserCar.SetFinished(true);
		if(m_cUserCar.GetPlace() == 1)
		{
			m_cUserCar.SetWonRace(true);
			SetWinningTime(m_cUserCar.GetTime());
		}
		m_bJustCrossedFinishLine = false;
	}
	return bCrash;
}

///////////////////////////////////////////////////////////////////
// CEnvironment::UpdateHighScoreScreen()
// Description:	updates high score screen environment
///////////////////////////////////////////////////////////////////
void CEnvironment::UpdateHighScoreScreen()
{
	for(int i = 0; i < NUM_ENEMY_CARS; i++)
	{
		m_acEnemyCars[i].Update((int*)this);
	}
}

///////////////////////////////////////////////////////////////////
// CEnvironment::UpdateGameStart()
// Description:	updates intro animation
///////////////////////////////////////////////////////////////////
void CEnvironment::UpdateGameStart()
{
	SetStartTime();
	m_cUserCar.SetStartTime(GetStartTime());
	UpdateGameStartCamera();
	if(m_fRotateCamera > 0.0f)
	{
		m_fRotateCamera -= 2.5f;
		m_fStartZ -= 0.3472f;
	}
}

///////////////////////////////////////////////////////////////////
// CEnvironment::UpdateGameStartCamera()
// Description:	updates the camera in game intro
///////////////////////////////////////////////////////////////////
void CEnvironment::UpdateGameStartCamera()
{
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glRotatef(m_fRotateCamera, 0.0f, 1.0f, 0.0f);
	gluLookAt(m_cUserCar.GetPos().data[0] + 25,
		m_cUserCar.GetPos().data[1], 
		m_cUserCar.GetPos().data[2] + 8.0f + m_fStartZ, 
		m_cUserCar.GetPos().data[0], 
		m_cUserCar.GetPos().data[1], 
		m_cUserCar.GetPos().data[2] + 7.0f + m_fStartZ, 
		0.0, 0.0, 1.0);  // set to track normal when we do 3D tracks
}

///////////////////////////////////////////////////////////////////
// CEnvironment::UpdateHighScoreCamera()
// Description:	updates high score screen camera
///////////////////////////////////////////////////////////////////
void CEnvironment::UpdateHighScoreCamera()
{
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(0.0f, -300.0f, 5.0f, -10.0f, -300.0f, 5.0f, 0.0, 0.0, 1.0);
}

///////////////////////////////////////////////////////////////////
// CEnvironment::Draw()
// Description:	draws the game world
///////////////////////////////////////////////////////////////////
void CEnvironment::Draw()
{
	m_cTrack.Draw();

	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);
	for(int i = 0; i < NUM_ENEMY_CARS; i++)
		m_acEnemyCars[i].Draw();
	m_cUserCar.Draw();
	glDisable(GL_CULL_FACE);
}

///////////////////////////////////////////////////////////////////
// CEnvironment::DrawHighScoreScreen()
// Description:	draw the high score screen
///////////////////////////////////////////////////////////////////
void CEnvironment::DrawHighScoreScreen()
{
	m_cTrack.Draw();
	for(int i = 0; i < NUM_ENEMY_CARS; i++)
		m_acEnemyCars[i].Draw();
}

///////////////////////////////////////////////////////////////////
// CEnvironment::LoadModels()
// Description:	load all the models needed for game environment
///////////////////////////////////////////////////////////////////
bool CEnvironment::LoadModels()
{
	for(int i = 0; i < NUM_ENEMY_CARS; i++)
	{
		switch(rand() % 7)
		{	
		case 0:
			m_acEnemyCars[i].Load("Models/porsche2.o.iv");
			break;
		case 1:
			m_acEnemyCars[i].Load("Models/porsche3.o.iv");
			break;
		case 2:
			m_acEnemyCars[i].Load("Models/porsche4.o.iv");
			break;
		case 3:
			m_acEnemyCars[i].Load("Models/porsche5.o.iv");
			break;
		case 4:
			m_acEnemyCars[i].Load("Models/porsche6.o.iv");
			break;
		case 5:
			m_acEnemyCars[i].Load("Models/porsche7.o.iv");
			break;
		case 6:
			m_acEnemyCars[i].Load("Models/porsche8.o.iv");
			break;
		default:
			m_acEnemyCars[i].Load("Models/porsche2.o.iv");
			break;
		}
	}
	m_cTrack.Load("Models/track.iv");
	m_cUserCar.Load("Models/porsche.o.iv");
	return true;
}

///////////////////////////////////////////////////////////////////
// CEnvironment::CalcCarPlace(CCar* pCar)
// Description:	calculate place of car in race
///////////////////////////////////////////////////////////////////
void CEnvironment::CalcCarPlace(CCar* pCar)
{
	int Place = 1;
	int i;
	for(i = 0; i < NUM_ENEMY_CARS; i++)
	{
		if(&m_acEnemyCars[i] != pCar)
		{
			if(m_acEnemyCars[i].GetLap() > pCar->GetLap())
				Place++;
			else if(m_acEnemyCars[i].GetLap() == pCar->GetLap() && m_acEnemyCars[i].GetQuadrant() > pCar->GetQuadrant())
				Place++;
			else if(m_acEnemyCars[i].GetLap() == pCar->GetLap() && m_acEnemyCars[i].GetQuadrant() == pCar->GetQuadrant())
			{
				switch(m_acEnemyCars[i].GetQuadrant())
				{
				case 0:
				case 5:
					if(m_acEnemyCars[i].GetPos().data[0] <= pCar->GetPos().data[0])
						Place++;
					break;
				case 1:
					if(m_acEnemyCars[i].GetPos().data[1] <= pCar->GetPos().data[1])
						Place++;
					break;
				case 2:
				case 3:
					if(m_acEnemyCars[i].GetPos().data[0] >= pCar->GetPos().data[0])
						Place++;
					break;
				case 4:
					if(m_acEnemyCars[i].GetPos().data[1] >= pCar->GetPos().data[1])
						Place++;
					break;
				}
			}
		}
	}
	if(&m_cUserCar != pCar)
	{
		if(m_cUserCar.GetLap() > pCar->GetLap())
			Place++;
		else if(m_cUserCar.GetLap() == pCar->GetLap() && m_cUserCar.GetQuadrant() > pCar->GetQuadrant())
			Place++;
		else if(m_cUserCar.GetLap() == pCar->GetLap() && m_cUserCar.GetQuadrant() == pCar->GetQuadrant())
		{
			switch(m_cUserCar.GetQuadrant())
			{
			case 0:
			case 5:
				if(m_cUserCar.GetPos().data[0] <= pCar->GetPos().data[0])
					Place++;
				break;
			case 1:
				if(m_cUserCar.GetPos().data[1] <= pCar->GetPos().data[1])
					Place++;
				break;
			case 2:
			case 3:
				if(m_cUserCar.GetPos().data[0] >= pCar->GetPos().data[0])
					Place++;
				break;
			case 4:
				if(m_cUserCar.GetPos().data[1] >= pCar->GetPos().data[1])
					Place++;
				break;
			}
		}
	}
	pCar->SetPlace(Place);
}