// CEnvironment Class Header
// 
// Description : container class for all objects in game world
//

#pragma once
// includes
#include "Track.h"
#include "UserCar.h"
#include "EnemyCar.h"

// number of enemy cars to race
#define NUM_ENEMY_CARS 8

///////////////////////////////////////////////////////////////////
// CEnvironment Class
// Description:	container class for all objects in game world
///////////////////////////////////////////////////////////////////
class CEnvironment
{
public:
	
					CEnvironment()						{}
	virtual			~CEnvironment()						{}
			
			// initialization functions for high score and game
			void	InitializeHighScoreScreen();
			void	InitializeEnvironment();

			// update functions for cameras, moving cars, pre-game animation, etc
			bool	Update(bool bUp, bool bDown, bool bLeft, bool bRight, UINT chKey);
			void	UpdateHighScoreScreen(void);
			void	UpdateHighScoreCamera(void);
			void	UpdateGameStartCamera(void);
			void	UpdateGameStart();

			// loading and drawing the game world
			bool	LoadModels(void);
			void	Draw(void);
			void	DrawHighScoreScreen(void);

			// timing, lap counting, and winner calculation
			void	SetStartTime()						{ m_fStartTime = (float)clock(); }
			float	GetStartTime() const				{ return m_fStartTime; }			
			void	CalcCarPlace(CCar* pCar);
			void	SetWinningTime(string time)			{ m_strWinningTime = time; }
			string	GetWinningTime() const				{ return m_strWinningTime; }
			void	SetNumLaps(int laps)				{ m_nNumLaps = laps; }
			int		GetNumLaps() const					{ return m_nNumLaps; }

// member variables
private:
	string		m_strWinningTime;
	float		m_fStartTime;
	int			m_nNumLaps;
	bool		m_bJustCrossedFinishLine;
	float		m_fStartZ;
	float		m_fRotateCamera;

public:
	CUserCar	m_cUserCar;
	CEnemyCar	m_acEnemyCars[NUM_ENEMY_CARS];
	CTrack		m_cTrack;
};