//		   Name: GU.h
//
//	Description: Main header file for the geomtry utilities library.
//
//		Version		Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		2.00		8/8/97		Sechrest	Conversion to new standard

#ifndef CPP_GEOMETRY_UTILITY_LIB

#define _LINT

#ifdef _DEBUG 
#pragma comment(lib, "GU_v2.2d.lib")
#else
#pragma comment(lib, "GU_v2.2.lib")
#endif

#pragma warning( push )
#pragma warning( disable : 4244)

#include <cstdlib>
#include <iostream>
#include <iomanip>
#include <cmath>

//
//	Constants
//

#define	GU_PI				3.14159265358979323846
#define	GU_PI_2	 			1.57079632679489661923
#define	GU_PI_4	 			0.78539816339744830962
#define	GU_MAX	 			3.4e+37F
#define	GU_MIN	 			1.17e-37F
#define	GU_SQRT2 			1.41421356237309504880
#define	GU_ROUND_TOLER		0.10
#define	GU_ANGLE_TOLER		0.05
#define	GU_NORMAL_TOLER		0.02
#define	GU_QUADRATIC_SPLINE	3
#define	GU_CUBIC_SPLINE		4
#define	GU_BEZIER_SPLINE	1
#define	GU_DONT_INTERSECT	0
#define	GU_DO_INTERSECT		1
#define	GU_COLINEAR			2
#define	GU_ERROR   			-1
#define	GU_SINGULAR			0
#define	GU_OK	   			1

//
//	Macros
//

#define	GURad2Deg(rad)			((rad) * 57.29578)
#define	GUDeg2Rad(deg)			((deg) / 57.29578)
#define	GUInterpolate(a, b, p)	(((b) - (a)) * (p) + (a))
#define	GURoughlyEqual(a, b, t)	(fabs((a) - (b)) < t)

#define	GUMidpoint(a, b)		((a + b) * 0.5)
#define	GUDistance(a, b)		(((a) - (b)).magnitude( ))
#define	GULongestExtent(a, b)	((((b) - (a)).abs( )).MaxAxis( ))
#define	GUShortestExtent(a, b)	((((b) - (a)).abs( )).MinAxis( ))
#define	GUAngleBetween(a, b)	(acosf(((a) ^ (b)) / ((a).magnitude( ) * (b).magnitude( ))))

class CMatrix;
class CBox;
class CPnt;

std::ostream	&operator<<(std::ostream &stream, CPnt &pnt);
std::ostream	&operator<<(std::ostream &stream, CBox &box);
std::ostream	&operator<<(std::ostream &stream, CMatrix &matrix);

//		   Name: CPnt
//
//	Description:
//
//		Version		Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		2.00		8/8/97		Sechrest	Conversion to new standard

class CPnt
{
private:
	int		count;	
	float	tolerance;

public:
	float	data[5];

// Constructors and Destructor
	CPnt( );
	CPnt(int c);
	CPnt(float a, float b);
	CPnt(float a, float b, float c);
	CPnt(float a, float b, float c, float d);
	CPnt(float a, float b, float c, float d, float e);
	~CPnt( );

// Functions
	void	set(float a)										{ data[0]=a; }
	void	set(float a, float b)								{ data[0]=a; data[1]=b; }
	void	set(float a, float b, float c)						{ data[0]=a; data[1]=b; data[2]=c; }
	void	set(float a, float b, float c, float d)				{ data[0]=a; data[1]=b; data[2]=c; data[3]=d; }
	void	set(float a, float b, float c, float d, float e)	{ data[0]=a; data[1]=b; data[2]=c; data[3]=d; data[4]=e; }
	float	GetTolerance( )										{ return tolerance; }
	void	SetTolerance(float t)								{ tolerance = t; }
	int		GetSize( )											{ return count; }
	void	SetSize(int nSize)									{ count = nSize; }

	float	magnitude( );
	float	MaxAxis( );
	float	MinAxis( );
	void	normalize( );
	int		RoughlyEqual(CPnt &pnt);
	CPnt	abs( );

// Operators
	friend std::ostream	&operator<<(std::ostream &stream, CPnt &pnt);
	CPnt			operator+(CPnt pnt);
	CPnt			operator-(CPnt pnt);
	CPnt			operator-( );
	float			operator^(CPnt &pnt);
	CPnt			operator*(CPnt &pnt);
	CPnt			operator*(float value);
	CPnt			operator*(CMatrix &matrix);
	CPnt			operator/(float value);
	int				operator==(CPnt &pnt);
	int				operator!=(CPnt &pnt);
	float&			operator[](int nIndex);
	int				operator<(CPnt &pnt){return (magnitude() < pnt.magnitude());}
	int				operator<=(CPnt &pnt){return (magnitude() <= pnt.magnitude());}
	int				operator>(CPnt &pnt){return (magnitude() > pnt.magnitude());}
	int				operator>=(CPnt &pnt){return (magnitude() >= pnt.magnitude());}
};



//		   Name: CMatrix
//
//	Description:
//
//		Version		Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		2.00		8/8/97		Sechrest	Conversion to new standard

class CMatrix
{
public:
	float	data[4][4];

// Constructors
	CMatrix( );

// Functions
	void	set(int i, int j, float value) { data[i][j] = value; }

	void	set(float m[4][4]);
	int		invert( );
	bool	IsIdentity( );
	void	LoadIdentity( );
	void	scale(float x);
	void	scale(float x, float y, float z);
	void	translate(float x, float y, float z);
	void	rotate(float angle, char axis);
	void	RotateReverse(float angle, char axis);
	void	RotateAxis(float angle, CPnt axis);

// Operators
	friend std::ostream	&operator<<(std::ostream &stream, CMatrix &matrix);
	CMatrix			operator*(CMatrix &matrix);
	CPnt			operator*(CPnt &pnt);
	CBox			operator*(CBox& cBox);
	CMatrix			operator+(CMatrix &matrix);
	CMatrix			operator-(CMatrix &matrix);
};



//		   Name: CBox
//
//	Description:
//
//		Version		Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		2.00		8/8/97		Sechrest	Conversion to new standard
#undef min
#undef max

class CBox
{
public:
	CPnt	min, max;

// Constructors
	CBox(int size = 3) : min(size), max(size)
	{
		Initialize( );
	}

// Functions
	float	Height( )			{ return max.data[1] - min.data[1]; }
	float	LongestExtent( )	{ return GULongestExtent(min, max); }
	CPnt	Midpoint( )			{ return GUMidpoint(min, max); }
	float	MidX( )				{ return (max.data[0] + min.data[0]) * 0.5; }
	float	MidY( )				{ return (max.data[1] + min.data[1]) * 0.5; }
	float	ShortestExtent( )	{ return GUShortestExtent(min, max); }
	float	Width( )			{ return max.data[0] - min.data[0]; }

	float	AverageExtent( );
	void	Initialize( );
	bool	IsSet( );
	void	ScaleByOrigin(float scale);
	float	SizeOfSquares(int count, int &rows, int &cols);

// Operators
	friend std::ostream	&operator<<(std::ostream &stream, CBox &box);
	CBox			operator+(CBox &box);
	CBox			operator+(CPnt &pnt);
	CBox			operator*(float scale);
	CBox			operator*(CMatrix& cMatrix);
	bool			operator<(CBox &box);
};


#ifdef	GU_LIB_CODE
#else

extern float	GUPnt2LineDist(CPnt &p, CPnt &a, CPnt &b);
extern int		GUParallelLines(CPnt &x1, CPnt &x2, CPnt &y1, CPnt &y2);
extern int		GUPerpLines(CPnt &x1, CPnt &x2, CPnt &y1, CPnt &y2);
extern int		GULinesX(CPnt &a, CPnt &b, CPnt &c, CPnt &d, CPnt &x);
extern void		GUAdjustAspect(int WindowWidth, int WindowHeight, CPnt &min, CPnt &max, CPnt &NewMin, CPnt &NewMax);
extern void		GUCalcAspect(int WindowWidth, int WindowHeight, CPnt &min, CPnt &max, CPnt &NewMin, CPnt &NewMax);
extern float	GUPnt2LineSegmentDistance(CPnt &p, CPnt &a, CPnt &b);
extern void		GUPerpBisector(CPnt &p, CPnt &a, CPnt &b, CPnt &x);
extern int		GUPntInBox(CPnt &pnt, CBox &box);
extern float	GUPnt2BoxDist(CPnt &pnt, CBox &box);

#endif	// GU_LIB_CODE

#define CPP_GEOMETRY_UTILITY_LIB

#pragma warning( pop )

#endif	// CPP_GEOMETRY_UTILITY_LIB

