// CGameEngine Class Header
// 
// Description:		top level class of game.
//					handles messages, input, game states, sound, music, and
//					basic program management
//
#pragma once

// includes
#include "OpenGLControlBase.h"
#include "3DModel.h"
#include "Environment.h"
#include "DSList.h"

// timer index
#define UPDATE_TIMER 1

// fading types
#define FADE_IN 0
#define FADE_OUT 1
#define FADE_NONE 3

///////////////////////////////////////////////////////////////////
// CGameEngine Class
// Description:	handles all game management
///////////////////////////////////////////////////////////////////
class CGameEngine : public COpenGLControlBase
{

public:
					CGameEngine();
	virtual			~CGameEngine();

// Attributes
public:
// Operations
public:
// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CGameEngine)
	//}}AFX_VIRTUAL

// Implementation
public:
	// Generated message map functions
protected:
	virtual void			vWindowKeyboardHandler(CKeyState cKey);
	virtual void			vWindowInitialExposure();
	virtual void			vWindowDraw();
	virtual void			vWindowUpdate();

	//{{AFX_MSG(CGameEngine)
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

private:
			void	DrawHighScores();
			void	DrawNewHighScores();
			void	DrawMainScreen();
			void	DrawAboutScreen();
			void	DrawStr(string strMessage, const GLuint dlMaterial = NULL);

			void	InitMainScreen();
			void	InitGameStart();
			void	InitGamePlay();
			void	InitGameClose();
			void	InitHighScore();


			void	LoadHighScores();
			void	SaveHighScores();
			int		CheckHighScore(string time);

	// member variables

	// game state management
	int						m_nNextGameState;
	int						m_nGameState;

	// environment
	CEnvironment			m_cEnv;

	// timing
	float					m_fIntroStartTime;
	float					m_fOutroStartTime;
	bool					m_bJustFinished;

	// high score variables
	string					m_strHighScores[7][2];
	int						m_nHighScorePlace;

	//keyboard stuff
	UINT					m_chLastKeyHit;
	bool					m_bUp;
	bool					m_bDown;
	bool					m_bLeft;
	bool					m_bRight;
	int						m_nHighScoreInitialsPosition;

	// mainscreen stuff
	int						m_nMenuItemSelected;
	GLfloat					m_fRotateStartScreenCar;

	// 3D Models
	C3DModel				m_cStartScreenCar;
	C3DModel				m_cPlatform;
	C3DModel				m_cMetalPlate;

	// Display Lists
	GLuint					m_dlFontLists;
	GLuint					m_dlBlueMaterial;
	GLuint					m_dlRedMaterial;
	GLuint					m_dlBack;

	// sound buffer list
	CDSList					m_cSound;

	// resolution changer
	CResolutionChanger		m_cResChanger;
	
	// fading variables
	int						m_nFadeType;
	float					m_fFade;
};
//{{AFX_INSERT_LOCATION}}
