//		   Name: Images.h
//
//	Description: 
//
//		This is a new library created specifically for Windows NT.
//		It is a combination of a few different libraries created for
//		the IRIX platform.  This library includes functionality for:
//
//			o Reading SGI/RGB image files
//			o Writing SGI/RGB image files
//			o Creating OpenGL textures from files
//			o Writing pixels in OpenGL
//
//		Future:
//			o Read and write BMP files (we'll need an image base class that
//			  BMP and SGI/RGB can be derived from, so that COpenGLImage
//			  can work with all formats).
//
//		Version		Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		00.01.00	04/07/99	King		Creation

#pragma once

//
//	Necessary header files
//
#include "Generic_v3.1.h"
#ifdef __sgi
#include <vector.h>
#else
#include <windows.h>
#include <wingdi.h>
#include <vector>
#endif // __sgi
#include <GL/gl.h>
#include <GL/glu.h>


//
//	Auto-linking feature
//
#ifdef _DEBUG
#pragma comment(lib, "Images_v1.3d.lib")
#else
#pragma comment(lib, "Images_v1.3.lib")
#endif // _DEBUG


typedef unsigned char Byte;
typedef Byte Rgba[4];

//		   Name: CSgiImage
//
//	Description:
//
//
//		Version 	Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		02.01.00	12/17/97	King		Creation
//		02.01.00	04/07/99	King		Ported to Windows NT

class CSgiImage : public CGeneric
{
public:
		CSgiImage( );
		CSgiImage(string strFilename);
		CSgiImage(int nWidth, int nHeight, int nDepth, Byte* pPixels);
		~CSgiImage( );

		short			GetRedPixel(int nCol, int nRow)	 				{ return short(m_pImageData[nRow * m_nWidth + nCol] >> 24) & 0xFF; }
		short			GetGreenPixel(int nCol, int nRow)				{ return short(m_pImageData[nRow * m_nWidth + nCol] >> 16) & 0xFF; }
		short			GetBluePixel(int nCol, int nRow)				{ return short(m_pImageData[nRow * m_nWidth + nCol] >>  8) & 0xFF; }
		short			GetAlphaPixel(int nCol, int nRow)				{ return short(m_pImageData[nRow * m_nWidth + nCol]) & 0xFF; }
		unsigned int*	GetOpenGLPixelFormat(void)						{ return (unsigned int*) m_pImageData; }

		void			SetRedPixel(int nCol, int nRow, short sValue) 	{ m_pImageData[nRow * m_nWidth + nCol] |= sValue >> 24; }
		void			SetGreenPixel(int nCol, int nRow, short sValue)	{ m_pImageData[nRow * m_nWidth + nCol] |= sValue >> 16; }
		void			SetBluePixel(int nCol, int nRow, short sValue)	{ m_pImageData[nRow * m_nWidth + nCol] |= sValue >>  8; }
		void			SetAlphaPixel(int nCol, int nRow, short sValue)	{ m_pImageData[nRow * m_nWidth + nCol] |= sValue; }

		int				GetHeight(void)									{ return m_nHeight; }
		int				GetWidth(void)									{ return m_nWidth; }
		int				GetDepth(void)									{ return m_nDepth; }

		void 			ReadFile(string strFilename);
		void			WriteFile(string strFilename);

		void			ConvertRGBAtoRGB(void);
private:
		bool			IsSgiFile(string strFilename);
		void			FreeImageData(void);
		void			SanityTest(void);

		bool			m_bDataIsExternal;
		int				m_nDepth;
		int				m_nHeight;
		int				m_nWidth;
		Byte* 			m_pImageData;
};



//		   Name: COpenGLImage
//
//	Description:
//
//
//		Version 	Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		02.01.00	12/19/97	King		Creation

class COpenGLImage : public CGeneric
{
public:
		enum EPosition
		{
			OGLI_LEFT, OGLI_RIGHT, OGLI_CENTER, OGLI_TOP, OGLI_BOTTOM
		};


		COpenGLImage( ) :
			m_bClamp(false)
		{
			m_pImage = NULL;	
			m_fScaleX = m_fScaleY = 1.0f;
		}

		COpenGLImage(string strFilename) :
			m_bClamp(false)
		{
			m_pImage = NULL;	
			COpenGLImage::ReadFile(strFilename);
			m_fScaleX = m_fScaleY = 1.0f;
		}

		~COpenGLImage( )
		{
			if (m_pImage)
			{
				delete m_pImage;
				m_pImage = NULL;
			}
		}

		CSgiImage*	GetImage(void)							{ return m_pImage; }
		int			GetHeight(void)							{ return m_pImage->GetHeight( ); }
		int			GetWidth(void)							{ return m_pImage->GetWidth( ); }
		int			GetDepth(void)							{ return m_pImage->GetDepth( ); }
		void		SetClamp(bool bFlag)					{ m_bClamp = bFlag; }
		void		SetScale(float fScale)					{ SetScale(fScale, fScale); }

		//		   Name: DrawPixels
		//
		//	Description:
		//
		//
		//		Version 	Date		Author		Comment	
		//	   ----------- ----------- ----------- -----------
		//		02.01.00	01/09/97	King		Creation

		void SetScale(float fScaleX, float fScaleY)
		{
			m_fScaleX = fScaleX;
			m_fScaleY = fScaleY;
			glPixelZoom(m_fScaleX, m_fScaleY);
		}


		//		   Name: DrawPixels
		//
		//	Description:
		//
		//
		//		Version 	Date		Author		Comment	
		//	   ----------- ----------- ----------- -----------
		//		02.01.00	12/19/97	King		Creation

		void DrawPixels(float fX, float fY, EPosition eHorizontal = OGLI_CENTER, EPosition eVertical = OGLI_CENTER)
		{
			GLint nViewport[4];
			GLdouble dProjectionMatrix[16];
			GLdouble dModelMatrix[16];
			double dX, dY, dZ;

			glGetDoublev(GL_PROJECTION_MATRIX, dProjectionMatrix);
			glGetDoublev(GL_MODELVIEW_MATRIX, dModelMatrix);
			glGetIntegerv(GL_VIEWPORT, nViewport);
			(void) gluProject(GLdouble(fX), GLdouble(fY), 0.0, dModelMatrix, dProjectionMatrix, nViewport, &dX, &dY, &dZ);

			glMatrixMode(GL_PROJECTION);
			glPushMatrix( );
				glLoadIdentity( );
				gluOrtho2D(0.0, GLdouble(nViewport[2]), 0.0, GLdouble(nViewport[3]));

				int nPosX, nPosY;
				switch (eHorizontal)
				{
				case OGLI_LEFT:
					nPosX = int(dX);
					break;
				case OGLI_RIGHT:
					nPosX = int(dX) - m_pImage->GetWidth( ) * m_fScaleX;
					break;
				default:
					nPosX = int(dX) - m_pImage->GetWidth( ) * m_fScaleX / 2;
				}
				switch (eVertical)
				{
				case OGLI_BOTTOM:
					nPosY = int(dY);
					break;
				case OGLI_TOP:
					nPosY = int(dY) - m_pImage->GetHeight( ) * m_fScaleY;
					break;
				default:
					nPosY = int(dY) - m_pImage->GetHeight( ) * m_fScaleY / 2;
				}

				glRasterPos2i(nPosX, nPosY);
				DrawPixels( );
			glPopMatrix( );
		}


		//		   Name: DrawPixels
		//
		//	Description:
		//
		//
		//		Version 	Date		Author		Comment	
		//	   ----------- ----------- ----------- -----------
		//		02.01.00	12/19/97	King		Creation

		void DrawPixels(bool bReducedColor = false)
		{
			if (!m_pImage)
				throw(usc_internal_error("COpenGLImage::DrawPixels - Cannot draw, no image has been sucessfully read."));
			glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

			switch (m_pImage->GetDepth( ))
			{
				case 4:
					glDrawPixels(m_pImage->GetWidth( ), m_pImage->GetHeight( ), bReducedColor ? GL_RGBA4 : GL_RGBA,
						GL_UNSIGNED_BYTE, (const GLvoid*) m_pImage->GetOpenGLPixelFormat( ));
					break;
				case 3:
					glDrawPixels(m_pImage->GetWidth( ), m_pImage->GetHeight( ), bReducedColor ? GL_RGB4 : GL_RGB,
						GL_UNSIGNED_BYTE, (const GLvoid*) m_pImage->GetOpenGLPixelFormat( ));
					break;
				case 2:
					glDrawPixels(m_pImage->GetWidth( ), m_pImage->GetHeight( ), GL_LUMINANCE_ALPHA,
						GL_UNSIGNED_BYTE, (const GLvoid*) m_pImage->GetOpenGLPixelFormat( ));
					break;
				case 1:
					glDrawPixels(m_pImage->GetWidth( ), m_pImage->GetHeight( ), GL_LUMINANCE,
						GL_UNSIGNED_BYTE, (const GLvoid*) m_pImage->GetOpenGLPixelFormat( ));
					break;
				default:
					throw(usc_internal_error(MakeString("COpenGLImage::DrawPixels - unknown image depth of [%d]", m_pImage->GetDepth( ))));
			}
		}


		//		   Name: MakeTexture
		//
		//	Description:
		//
		//
		//		Version 	Date		Author		Comment	
		//	   ----------- ----------- ----------- -----------
		//		02.01.00	12/19/97	King		Creation

		GLuint MakeTexture(bool bReducedColor = false, GLint nMagFilter = GL_LINEAR, GLint nMinFilter = GL_LINEAR_MIPMAP_LINEAR)
		{
			if (!m_pImage)
				throw(usc_internal_error("COpenGLImage::MakeTexture - Cannot draw, no image has been sucessfully read."));

			GLuint uiTextureIndex;
			glGenTextures(1, &uiTextureIndex);
			glBindTexture(GL_TEXTURE_2D, uiTextureIndex);
			glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

			if (m_bClamp)
			{
				glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
				glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
			}
			else
			{
				glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
				glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
			}

			glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, nMagFilter);
			glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, nMinFilter);

			switch (m_pImage->GetDepth( ))
			{
				case 4:
					gluBuild2DMipmaps(GL_TEXTURE_2D, bReducedColor ? GL_RGBA : GL_RGBA,
						m_pImage->GetWidth( ), m_pImage->GetHeight( ), GL_RGBA,
						GL_UNSIGNED_BYTE, (const void*) m_pImage->GetOpenGLPixelFormat( ));
					break;
				case 3:
					gluBuild2DMipmaps(GL_TEXTURE_2D, bReducedColor ? GL_RGB: GL_RGB,
						m_pImage->GetWidth( ), m_pImage->GetHeight( ), GL_RGB,
						GL_UNSIGNED_BYTE, (const void*) m_pImage->GetOpenGLPixelFormat( ));
					break;
				case 2:
					gluBuild2DMipmaps(GL_TEXTURE_2D, GL_LUMINANCE_ALPHA,
						m_pImage->GetWidth( ), m_pImage->GetHeight( ), GL_LUMINANCE_ALPHA,
						GL_UNSIGNED_BYTE, (const void*) m_pImage->GetOpenGLPixelFormat( ));
					break;
				case 1:
					gluBuild2DMipmaps(GL_TEXTURE_2D, GL_LUMINANCE,
						m_pImage->GetWidth( ), m_pImage->GetHeight( ), GL_LUMINANCE,
						GL_UNSIGNED_BYTE, (const void*) m_pImage->GetOpenGLPixelFormat( ));
					break;
				default:
					throw(usc_internal_error(MakeString("COpenGLImage::MakeTexture - unknown image depth of [%d]", m_pImage->GetDepth( ))));
			}

			return uiTextureIndex;
		}


		//		   Name: ReadFile
		//
		//	Description:
		//
		//
		//		Version 	Date		Author		Comment	
		//	   ----------- ----------- ----------- -----------
		//		02.01.00	12/19/97	King		Creation

		void ReadFile(string strFilename)
		{
			if (m_pImage)
			{
				delete m_pImage;
				m_pImage = NULL;
			}

			m_pImage = new CSgiImage;
			try
			{
				m_pImage->ReadFile(strFilename);
			}
			catch (...)
			{
				delete m_pImage;
				m_pImage = NULL;
				throw;
			}
		}

private:
		bool		m_bClamp;
		CSgiImage*	m_pImage;
		float		m_fScaleX;
		float		m_fScaleY;
};
