/*--BEGIN FILE--(invparse.h)-------------------------------------------------*/
/*
!
!	USC IRIS Inventor 1.0 Parser		Started: 21 July 1994
!
!
*/

#pragma once

#ifndef __sgi
//
//	Auto-linking feature
//
#ifdef _DEBUG
#pragma comment(lib, "InventorParser_v3.0d.lib")
#else
#pragma comment(lib, "InventorParser_v3.0.lib")
#endif // _DEBUG
#endif // not __sgi

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <ctype.h>
#include <string.h>
#include <limits.h>
#include <stdarg.h>
#ifndef __sgi
#include <windows.h>
#include <wingdi.h>
#include <iostream>
#else
#include <iostream.h>
#endif // not __sgi
#include <setjmp.h>
#include <GL/gl.h>
#include <GL/glu.h>

#ifndef FREE_SCENE
#define	FREE_SCENE
#endif

#define	INV_UNDEFINED	-1
#define _LINT

//
//	The old style Geometry Utilities library is embedded here.  This was
//	written in C and is very nasty.  Nobody should ever have to be
//	subjected to this header file...
//

/*---CONSTANTS---------------------------------------------------------------*/

#define GU_PI			3.14159265358979323846
#define GU_PI_2			1.57079632679489661923
#define GU_PI_4			0.78539816339744830962
#define	GU_MAX			3.4e+37F
#define	GU_MIN			1.17e-37F
#define	GU_SQRT2		1.41421356237309504880
#define	GU_ROUND_TOLER		0.10
#define	GU_QUADRATIC_SPLINE	3
#define	GU_CUBIC_SPLINE		4
#define	GU_BEZIER_SPLINE	1
enum { xGU_DONT_INTERSECT, xGU_DO_INTERSECT, xGU_COLINEAR };

/*---MACROS------------------------------------------------------------------*/

#define GULookupHSItoRGB(h, s, i)               \
        HSItoRGBTable[h >> GULookupShift]       \
                     [s >> GULookupShift]       \
                     [i >> GULookupShift]

#define	GUCopyMatrix(a, b)		memcpy(a, b, sizeof(GUMatrix))
#define	GUCopyPoint2D(a, b)		memcpy(a, b, sizeof(Point2D))
#define	GUCopyPoint3D(a, b)		memcpy(a, b, sizeof(Point3D))
#define	GUCopyPoint4D(a, b)		memcpy(a, b, sizeof(Point4D))
#define	GUCopyPoint5D(a, b)		memcpy(a, b, sizeof(Point5D))
#define	GUSetPoint2D(p, a, b)		(p)[0] = (a); (p)[1] = (b)
#define	GUSetPoint3D(p, a, b, c)	(p)[0] = (a); (p)[1] = (b);	\
					(p)[2] = (c)
#define	GUSetPoint3DTo0s(a)		(a)[0] = (a)[1] = (a)[2] = 0.0
#define	GUSetPoint2DTo0s(a)		(a)[0] = (a)[1] = 0.0
#define	GUSetPoint3DTo1s(a)		(a)[0] = (a)[1] = (a)[2] = 1.0
#define	GUSetPoint2DTo1s(a)		(a)[0] = (a)[1] = 1.0
#define	GUSetAllPoint3D(a, v)		(a)[0] = (a)[1] = (a)[2] = (v)
#define	GUSetAllPoint2D(a, v)		(a)[0] = (a)[1] = (v)
#define	GUPrintPoint3D(p)		printf("(%g, %g, %g)\n",	\
						(p)[0], (p)[1], (p)[2]);
#define	GUPrintPoint2D(p)		printf("(%g, %g)\n",	\
						(p)[0], (p)[1]);
#define	GUPointsEqual2D(a, b)		(((a)[0] == (b)[0]) && \
				 	 ((a)[1] == (b)[1]))
#define	GUPointsEqual3D(a, b)		(((a)[0] == (b)[0]) && \
				 	 ((a)[1] == (b)[1]) && \
				 	 ((a)[2] == (b)[2]))
#define	GUSubVectors2D(a, b, c)		(c)[0] = (b)[0] - (a)[0]; \
					(c)[1] = (b)[1] - (a)[1]
#define	GUSubVectors3D(a, b, c)		(c)[0] = (b)[0] - (a)[0]; \
					(c)[1] = (b)[1] - (a)[1]; \
					(c)[2] = (b)[2] - (a)[2]
#define	GUAddVectors2D(a, b, c)		(c)[0] = (b)[0] + (a)[0]; \
					(c)[1] = (b)[1] + (a)[1]
#define	GUAddVectors3D(a, b, c)		(c)[0] = (b)[0] + (a)[0]; \
					(c)[1] = (b)[1] + (a)[1]; \
					(c)[2] = (b)[2] + (a)[2]
#define	GUScaleVector2D(a, s)		(a)[0] *= s; \
					(a)[1] *= s
#define	GUScaleVector3D(a, s)		(a)[0] *= s; \
					(a)[1] *= s; \
					(a)[2] *= s
#define	GUTranslatePoint3D(a, t)	(a)[0] += (t)[0]; \
					(a)[1] += (t)[1]; \
					(a)[2] += (t)[2]
#define	GUMagnitude3D(v)		sqrt((v)[0] * (v)[0] + \
					     (v)[1] * (v)[1] + \
					     (v)[2] * (v)[2])
#define	GUNegatePoint3D(p)		(p)[0] = -(p)[0];	\
					(p)[1] = -(p)[1];	\
					(p)[2] = -(p)[2]
#define	xGURoughlyEqual(a, b)		(fabs((a) - (b)) < GU_ROUND_TOLER)
#define	GURoughlyEqualTol(a, b, t)	(fabs((a) - (b)) < t)
#define	GUPointsRoughlyEqual2D(a, b)	(GURoughlyEqual(a[0], b[0]) && \
					 GURoughlyEqual(a[1], b[1]))

#define	GUPointsRoughlyEqual2DTol(a, b, t) \
					(GURoughlyEqualTol(a[0], b[0], t) && \
					 GURoughlyEqualTol(a[1], b[1], t))

#define	GUPointsRoughlyEqual3D(a, b)	(GURoughlyEqual(a[0], b[0]) && \
					 GURoughlyEqual(a[1], b[1]) && \
					 GURoughlyEqual(a[2], b[2]))

#define	GUPointsRoughlyEqual3DTol(a, b, t) \
					(GURoughlyEqualTol(a[0], b[0], t) && \
					 GURoughlyEqualTol(a[1], b[1], t) && \
					 GURoughlyEqualTol(a[2], b[2], t))
#define	GUMidpnt2D(a, b, c)		(c)[0] = ((a)[0] + (b)[0]) / 2.0; \
					(c)[1] = ((a)[1] + (b)[1]) / 2.0;
#define	GUMidpnt3D(a, b, c)		(c)[0] = ((a)[0] + (b)[0]) / 2.0; \
					(c)[1] = ((a)[1] + (b)[1]) / 2.0; \
					(c)[2] = ((a)[2] + (b)[2]) / 2.0;
#define	GUDeg2Rad(deg)			((deg) / 57.29578)
#define GURad2Deg(rad)			((rad) * 57.29578)
#define	GUDistance2D(a, b)		sqrt((((a)[0] - (b)[0])  * \
					      ((a)[0] - (b)[0])) + \
					     (((a)[1] - (b)[1])  * \
					      ((a)[1] - (b)[1])))
#define	GUDistanceF2D(a, b)		sqrtf((((a)[0] - (b)[0])  * \
					      ((a)[0] - (b)[0])) + \
					     (((a)[1] - (b)[1])  * \
					      ((a)[1] - (b)[1])))
#define	GUDistance3D(a, b)		sqrt((((a)[0] - (b)[0])  * \
					      ((a)[0] - (b)[0])) + \
					     (((a)[1] - (b)[1])  * \
					      ((a)[1] - (b)[1])) + \
					     (((a)[2] - (b)[2])  * \
					      ((a)[2] - (b)[2])))
#define	GUDistanceF3D(a, b)		sqrtf((((a)[0] - (b)[0])  * \
					      ((a)[0] - (b)[0])) + \
					     (((a)[1] - (b)[1])  * \
					      ((a)[1] - (b)[1])) + \
					     (((a)[2] - (b)[2])  * \
					      ((a)[2] - (b)[2])))
#define	GUDotProduct(a, b)		(a[0] * b[0] + a[1] * b[1] + \
					 a[2] * b[2])
#define	xGUInterpolate(a, b, p)		((a) + ((b) - (a)) * (p))
#define	GUInterpolate2D(a, b, p)	GUInterpolate((a)[0], (b)[0], (p)); \
					GUInterpolate((a)[1], (b)[1], (p))
#define	GUInterpolate3D(a, b, p)	GUInterpolate((a)[0], (b)[0], (p)); \
					GUInterpolate((a)[1], (b)[1], (p)); \
					GUInterpolate((a)[2], (b)[2], (p))

/*---STRUCTURES AND TYPES----------------------------------------------------*/

typedef	float	GUMatrix[4][4];
typedef	float	Point2D[2];
typedef	float	Point3D[3];
typedef	float	Point4D[4];
typedef	float	Point5D[5];

/*
!	Only for the Geometry Utility Library
*/

#ifdef GEOMUTIL_LIB_CODE

/*--STATE DEFINITIONS--------------------------------------------------------*/

#define	TRUE		1
#define	FALSE		0

/*--MACROS-------------------------------------------------------------------*/

/*--STRUCTURES---------------------------------------------------------------*/

#endif

/*--IMPORTING FUNCTIONS------------------------------------------------------*/

/*
!	Only for application code
*/

#ifndef GEOMUTIL_LIB_CODE

extern	void	GUMatLoadId(GUMatrix);
extern	void	GUMatScale(GUMatrix, float, float, float);
extern	void	GUMatTranslate(GUMatrix, float, float, float);
extern	void	GUMatRot(GUMatrix, float, char);
extern	void	GUMatRotReverse(GUMatrix, float, char);
extern	void	GUMatAxisRot(GUMatrix, Point3D, float);
extern	void	GUMatMultiply(GUMatrix, GUMatrix);
extern	void	GUMatPntMultiply(GUMatrix, Point3D);
extern	void	GUPntMatMultiply(GUMatrix, Point3D);
extern	void	GUMatPrint(GUMatrix);

extern	double	GUAngleBetweenPnts(Point2D, Point2D, Point2D);
extern	float	GUPnt2LineDist(Point2D, Point2D, Point2D);
extern	void	GURotAroundPnt2D(Point2D, Point2D, double, Point2D);
extern	int	GUParallelLines(Point2D, Point2D, Point2D, Point2D);
extern	int	GUPerpLines(Point2D, Point2D, Point2D, Point2D);
extern	int	GULinesX(Point2D, Point2D, Point2D, Point2D, Point2D);
extern	void	GUNormalize2D(Point2D);
extern	void	GUAdjustAspect(int, int, Point2D, Point2D, Point2D, Point2D);
extern	void	GUCalcAspect(int, int, Point2D, Point2D, Point2D, Point2D);

extern  long    GUHSItoRGB(float, float, float, int);
extern  int     GUBuildHSItoRGBTable(int, int);

extern	float	xGULongestExtent(Point3D, Point3D);
extern	float	GUMaxAxis(Point3D);
extern	void	GUMakeVector3D(Point3D, Point3D, Point3D);
extern	void	GUCrossProduct(Point3D, Point3D, Point3D, Point3D);
extern	void	GUCrossProductVectors(Point3D, Point3D, Point3D);
extern	void	GUNormalize3D(Point3D);

extern	void	GULinearInterp(float *, int, float *, int, int);
extern	int	GUCatmullRom(Point3D *, int, Point3D *, int);
extern	void	GUBSpline(int, float *, int, float *, int, int);

extern	Point2D	GUOnes2D, GUZeros2D;
extern	Point3D	GUOnes3D, GUZeros3D;
extern	Point5D	GUOnes5D, GUZeros5D;

extern  long    ***HSItoRGBTable;
extern  int     GULookupShift;

#endif


//
//	The old style Linked List library is embedded here.  This was
//	written in C and is very nasty.  Nobody should ever have to be
//	subjected to this header file...
//
#ifndef LINKED_LIST_LIB

#define _LINT
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <sys/types.h>

#define	LL_POINTERS	0
#define	LL_UNDEFINED	-2

#ifdef LL_LIBCODE

/*--STATE DEFINITIONS--------------------------------------------------------*/

#define	LL_FORWARD	1
#define	LL_BACKWARD	-1

/*--MACROS-------------------------------------------------------------------*/

/*--STRUCTURES---------------------------------------------------------------*/

#endif

typedef struct NodeStruct {
	void			*data;
	struct	NodeStruct	*prev, *next;
} ListNode;
typedef ListNode *LLnode;

typedef struct {
	int		NodeCount, LastReadIndex, LastWriteIndex;
	ListNode	*first, *last, *LastReadPtr, *LastWritePtr;
	size_t		NodeSize;
	void		(*destroy) (void *);
} LinkedList;
typedef LinkedList *LLlist;

/*--IMPORTING FUNCTIONS------------------------------------------------------*/

#ifndef LL_LIBCODE

extern  LLlist  LLCreateList(size_t);
extern  void	LLDestroyList(LLlist);
extern	void	LLDestroyFunction(LLlist, void (*)( ));

extern  LLnode  LLCreateNode(LLlist);
extern  void    LLAppend(LLlist, void *);
extern  void    LLWriteover(LLlist, int, void *);
extern  void    LLInsert(LLlist, int, void *);
extern	int	LLAddUnique(LLlist, void *, int (*)( ));
extern	int	LLSearch(LLlist, void *, int (*)( ));
extern	int	LLSearchPntr(LLlist, void *);

extern  void    LLDelete(LLlist, int);
extern  void    LLRemove(LLlist, int);
extern  void    LLEraseList(LLlist);
extern	LLlist	LLCopyListPointers(LLlist);
extern	LLlist	LLCopyListContents(LLlist);
extern  void    LLRemoveList(LLlist);
extern  void    *LLRetrieve(LLlist, int);
extern	void	*LLNewAppend(LLlist);
extern  void    *LLMalloc(LLlist);

extern	int	LLCompareInt(int *, int *);
extern	int	LLCompareCharString(char *, char *);
extern	int	LLComparePoint3D(float *, float *);

#define	LLFirst(list)	LLRetrieve(list, 0)
#define	LLLast(list)	LLRetrieve(list, list->NodeCount - 1)

#endif

#define	LINKED_LIST_LIB

#endif

/*--STATE DEFINITIONS--------------------------------------------------------*/

#ifdef LIBRARY_CODE
//#include <CStyleGeometryUtilities_v1.0.h>
#define	TRUE		1
#define	FALSE		0
#endif

#define	INV_ERROR	-1
#define	INV_OK		0
#define BB_RESET_LOCAL	0
#define BB_RESET_GLOBAL	1

#define	INV_SHOW_TRISTRIPS	1
#define	INV_SHOW_FACESETS	2
#define	INV_SHOW_SPHERES	4
#define	INV_SHOW_LINES		8

/*--ENUMERATING DEFINITIONS--------------------------------------------------*/

enum 
	{
	INV_PICK_NONE, INV_PICK_GROUP, INV_PICK_POLYGON
	};

/*--LIMITING DEFINITIONS-----------------------------------------------------*/

#ifdef LIBRARY_CODE
#define	MAX_STR_LEN	8192
#define	MAX_SYMBOLS	30
#endif

/*--OTHER DEFINITIONS--------------------------------------------------------*/

#define	INV_PI		3.14159265358979323846
#define INV_PI_2	1.57079632679489661923
#define INV_PI_4	0.78539816339744830962
#define	INV_NO_LEVEL	-1

/*--MACROS-------------------------------------------------------------------*/

#define	DEG2RAD(rad)    	((rad) / 57.29578)
#define RAD2DEG(rad)            ((rad) * 57.29578)
#define	INVCopyPoint2D(a, b)	memcpy(a, b, sizeof(INVPoint2D))
#define	INVCopyPoint3D(a, b)	memcpy(a, b, sizeof(INVPoint3D))

#ifdef LIBRARY_CODE
#define	INDEX2D(ptr, r, c)	(*(ptr + r * 4 + c))

#define	BindTexture(index)	if (index != 0) texbind(TX_TEXTURE_0, index);
#define	IsGroupNode(n) 		(n >= ArraySym && n <= SwitchSym)
#define	IsShapeNode(n) 		(n >= ConeSym && n <= VertexShapeSym)
#define	IsPropertyNode(n) 	(n >= BaseColorSym && n <= UnitsSym)
#define	IsLightNode(n) 		(n >= DirectionalLightSym && n <= SpotLightSym)
#define	IsCameraNode(n) 	(n >= CameraSym && n <= PerspectiveCameraSym)
#endif

/*--STRUCTURES---------------------------------------------------------------*/

typedef float	INVMatrixType[4][4];
typedef float	INVPoint4D[4];
typedef float 	INVPoint3D[3];
typedef float 	INVPoint2D[2];
typedef long	INVObject;

#ifdef LIBRARY_CODE

typedef struct
    {
    float   alpha;
    Point3D ambient;
    Point3D diffuse;
    Point3D emission;
    float   shininess;
    Point3D specular;
    } GLUMaterialType;

typedef struct
	{
        char    filename[MAX_STR_LEN];
	} INVUsedTexType;

typedef struct
	{
	int	node, count, symbols[MAX_SYMBOLS];
	} ParseInfo;

typedef struct
	{
        INVPoint3D TexMin, TexMax, CoordMin, CoordMax;
	int	mylarge, mymedium, mysmall;
	float	LargeDist, MedDist, ratio;
	} INVBoundingBoxType;

typedef struct
	{
	int	TextureEnabled;
	} OpenGLStateType;

#endif

/*--ENUMURATIONS-------------------------------------------------------------*/

enum 
	{ 
	TEV_DECAL_INDEX = 760, TEV_MODULATE_INDEX, TEV_BLEND_INDEX
	};

#ifdef LIBRARY_CODE
enum 
	{ 
	Color, Enum, Float, Vec2f, 
	Vec3f, Vec4f, Short, 
	UShort, Name, StringType, Long, 
	ULong, MatrixType, BitMask, 
	Rotation, Path, LongArray,
	Vec2fArray, Vec3fArray, Vec4fArray,
	StringArray, FloatArray, ULongArray,
	NumTypes
	};

static	int 	TypeSizes[NumTypes] = 
	{
	sizeof(INVPoint3D), sizeof(int), sizeof(float), sizeof(INVPoint2D),
	sizeof(INVPoint3D), sizeof(INVPoint4D), sizeof(short), 
	sizeof(unsigned short), sizeof(char *), sizeof(char *), sizeof(long), 
	sizeof(unsigned long), sizeof(INVMatrixType), sizeof(long),
	sizeof(INVPoint4D), sizeof(char *), sizeof(long *),
	sizeof(INVPoint2D *), sizeof(INVPoint3D *), sizeof(INVPoint4D *),
	sizeof(char **), sizeof(float *), sizeof(unsigned long *),
	};
#endif

#ifdef LIBRARY_CODE

/*--INVENTOR NODE STRUCTURES-------------------------------------------------*/

/*
!       Shape Node Classes
*/

typedef struct
	{
	long	parts;
	float	bottomRadius;
	float	height;
	} INVConeType;

typedef struct
	{
	float	width;
	float	height;
	float	depth;
	} INVCubeType;

typedef struct
	{
	long	parts;
	float	radius;
	float	height;
	} INVCylinderType;

typedef struct
	{
	long	startIndex;
	long	*numVertices;
	} INVFaceSetType;

typedef struct
	{
	long	*coordIndex;
	long	*materialIndex;
	long	*normalIndex;
	long	*textureCoordIndex;
	int	count;
	} INVIndexedFaceSetType;

typedef struct
	{
	long	*coordIndex;
	long	*materialIndex;
	long	*normalIndex;
	long	*textureCoordIndex;
	int	count;
	} INVIndexedLineSetType;

typedef struct
	{
	long	*coordIndex;
	float	*knotVector;
	} INVIndexedNurbsCurveType;

typedef struct
	{
	long	numUControlPoints;
	long	numVControlPoints;
	long	numSControlPoints;
	long	numTControlPoints;
	long	*coordIndex;
	float	*uKnotVector;
	float	*vKnotVector;
	float	*sKnotVector;
	float	*tKnotVector;
	long	*textureCoordIndex;
	} INVIndexedNurbsSurfaceType;

typedef struct
	{
	int	dummy;
	} INVIndexedShapeType;

typedef struct
	{
	long	*coordIndex;
	long	*materialIndex;
	long	*normalIndex;
	long	*textureCoordIndex;
	} INVIndexedTriangleStripSetType;

typedef struct
	{
	long	startIndex;
	long	*numVertices;
	} INVLineSetType;

typedef struct
	{
	int	dummy;
	} INVNonIndexedShapeType;

typedef struct
	{
	long	numControlPoints;
	float	*knotVector;
	} INVNurbsCurveType;

typedef struct
	{
	long	numUControlPoints;
	long	numVControlPoints;
	long	numSControlPoints;
	long	numTControlPoints;
	float	*uKnotVector;
	float	*vKnotVector;
	float	*sKnotVector;
	float	*tKnotVector;
	} INVNurbsSurfaceType;

typedef struct
	{
	long	startIndex;
	long	numPoints;
	} INVPointSetType;

typedef struct
	{
	long	startIndex;
	long	verticesPerColumn;
	long	verticesPerRow;
	} INVQuadMeshType;

typedef struct
	{
	int	dummy;
	} INVShapeType;

typedef struct
	{
	float	radius;
	} INVSphereType;

typedef struct
	{
	char	**string;
	float	spacing;
	int	justification;
	} INVText2Type;

typedef struct
	{
	char	**string;
	float	spacing;
	int	justification;
	long	parts;
	} INVText3Type;

typedef struct
	{
	long	startIndex;
	long	*numVertices;
	} INVTriangleStripSetType;

/*
!       Property Node Classes
*/

typedef struct
	{
	INVPoint3D	*rgb;
	} INVBaseColorType;

typedef struct
	{
	int	type;
	float	value;
	} INVComplexityType;

typedef struct
	{
	INVPoint3D	*point;
	} INVCoordinate3Type;

typedef struct
	{
	INVPoint4D	*point;
	} INVCoordinate4Type;

typedef struct
	{
	int		style;
	short		lineWidth;
	unsigned short	linePattern;
	} INVDrawStyleType;

typedef struct
	{
	float		ambientIntensity;
	INVPoint3D	ambientColor;
	INVPoint3D	attenuation;
	int		fogType;
	int		fogComputed;
	INVPoint3D	fogColor;
	float		fogDensity;
	float		fogNearDistance;
	float		fogFarDistance;
	} INVEnvironmentType;

typedef struct
	{
	char	*name;
	float	size;
	} INVFontType;

typedef struct
	{
	char	*string;
	} INVInfoType;

typedef struct
	{
	char	*label;
	} INVLabelType;

typedef struct
	{
	int		model;
	INVPoint3D	nearColor;
	INVPoint3D	farColor;	
	} INVLightModelType;

typedef struct
	{
	long	*index;
	int	linkage;
	} INVLinearProfileType;

typedef struct
	{
	INVPoint3D	ambientColor;
	INVPoint3D	diffuseColor;
	INVPoint3D	specularColor;
	INVPoint3D	emissiveColor;
	float		shininess;
	float		transparency;
	} INVMaterialType;

typedef struct
	{
	int	value;
	} INVMaterialBindingType;

typedef struct
	{
	INVMatrixType	matrix;
	} INVMatrixTransformType;

typedef struct
	{
	INVPoint3D	*vector;
	} INVNormalType;

typedef struct
	{
	int	value;
	} INVNormalBindingType;

typedef struct
	{
	float	*knotVector;
	long	*index;
	int	linkage;
	} INVNurbsProfileType;

typedef struct
	{
	unsigned long	*rgba;
	} INVPackedColorType;

typedef struct
	{
	int	style;
	} INVPickStyleType;

typedef struct
	{
	INVPoint2D	*point;
	} INVProfileCoordinate2Type;

typedef struct
	{
	INVPoint3D	*point;
	} INVProfileCoordinate3Type;

typedef struct
	{
	long	whatToReset;
	} INVResetTransformType;

typedef struct
	{
	INVPoint4D	rotation;	
	} INVRotationType;

typedef struct
	{
	int	axis;
	float	angle;
	} INVRotationXYZ;

typedef struct
	{
	INVPoint3D	scaleFactor;
	} INVScaleType;

typedef struct
	{
	long	hints;
	float	creaseAngle;
	long	vertexOrdering;
	long	faceType;
	long	shapeType;
	} INVShapeHintsType;

typedef struct
	{
	long		component;
	char		*filename;
	int		minFilter;
	int		magFilter;
	int		wrapS;
	int		wrapT;
	int		model;
	INVPoint3D	blendColor;
	INVPoint2D	translation;
	INVPoint2D	scaleFactor;
	float		rotation;
	INVPoint2D	center;
	} INVTexture2Type;

typedef struct
	{
	INVPoint2D	translation;
	float		rotation;
	INVPoint2D	scaleFactor;
	INVPoint2D	center;
	} INVTexture2TransformType;

typedef struct
	{
	INVPoint2D	*point;
	} INVTextureCoordinate2Type;

typedef struct
	{
	int	value;
	} INVTextureCoordinateBindingType;

typedef struct
	{
	int		coord;
	INVPoint3D	center;
	INVPoint4D	rotation;
	} INVTextureCoordinateCubeType;

typedef struct
	{
	int		coord;
	INVPoint3D	center;
	INVPoint4D	rotation;
	float	height;
	} INVTextureCoordinateCylinderType;

typedef struct
	{
	int	coord;
	} INVTextureCoordinateEnvironmentType;

typedef struct
	{
	int	dummy;
	} INVTextureCoordinateFunctionType;

typedef struct
	{
	INVPoint3D	directionS;
	INVPoint3D	directionT;
	/*
	int		coord;
	INVPoint3D	normal;
	float		distanceFromOrigin;
	float		repeatInterval;
	*/
	} INVTextureCoordinatePlaneType;

typedef struct
	{
	int		coord;
	INVPoint3D	center;
	INVPoint4D	rotation;
	} INVTextureCoordinateSphereType;

typedef struct
	{
	INVPoint3D	translation;
	INVPoint4D	rotation;
	INVPoint3D	scaleFactor;
	INVPoint4D	scaleOrientation;
	INVPoint3D	center;
	} INVTransformType;

typedef struct
	{
	INVPoint3D	translation;
	} INVTranslationType;

typedef struct
	{
	int	units;
	} INVUnitsType;

/*
!       Group Node Classes
*/

typedef struct
	{
	unsigned short	numElements1;
	unsigned short	numElements2;
	unsigned short	numElements3;
	INVPoint3D	separation1;
	INVPoint3D	separation2;
	INVPoint3D	separation3;
	int		origin;
	} INVArrayType;

typedef struct
	{
	char	**className;
	char	**fields;
	char	**customData;
	} INVCustomNodeType;

typedef struct
	{
	char	*name;
	} INVFileType;

typedef struct
	{
	int	dummy;
	} INVGroupType;

typedef struct
	{
	int	dummy;
	} INVLayerGroupType;

typedef struct
	{
	INVMatrixType	*matrix;
	} INVMultipleCopyType;

typedef struct
	{
	char	*path;
	} INVPathSwitchType;

typedef struct
	{
	int	dummy;
	} INVSelectionType;

typedef struct
	{
	int	dummy;
	} INVSeparatorType;

/*
!       Light Node Classes
*/

typedef struct
	{
	float		intensity;
	INVPoint3D	color;
	INVPoint3D	direction;
	} INVDirectionalLightType;

typedef struct
	{
	int	dummy;
	} INVLightType;

typedef struct
	{
	float		intensity;
	INVPoint3D	color;
	INVPoint3D	location;
	} INVPointLightType;

typedef struct
	{
	float		intensity;
	INVPoint3D	color;
	INVPoint3D	location;
	INVPoint3D	direction;
	float		dropOffRate;
	float		cutOffAngle;
	} INVSpotLightType;

/*
!       Camera Node Classes
*/

typedef struct
	{
	int	dummy;
	} INVCameraType;

typedef struct
	{
	int		viewportMapping;
	INVPoint3D	position;
	INVPoint4D	orientation;
	float		aspectRatio;
	float		nearDistance;
	float		farDistance;
	float		focalDistance;
	float		height;	
	} INVOrthographicCamera;

typedef struct
	{
	int	viewportMapping;
	INVPoint3D	position;
	INVPoint4D	orientation;
	float	aspectRatio;
	float	nearDistance;
	float	farDistance;
	float	focalDistance;
	float	heightAngle;	
	} INVPerspectiveCamera;

#endif

/*--GENERIC NODE STRUCTURE---------------------------------------------------*/

#ifdef LIBRARY_CODE
typedef union
	{
	/*
	!	Shape Node Classes
	*/

	INVConeType				cone;
	INVCubeType				cube;
	INVCylinderType				cylinder;
	INVFaceSetType				faceSet;
	INVIndexedFaceSetType			indexedFaceSet;
	INVIndexedLineSetType			indexedLineSet;
	INVIndexedNurbsCurveType		indexedNurbsCurve;
	INVIndexedNurbsSurfaceType		indexedNurbsSurface;
	INVIndexedShapeType			indexedShape;
	INVIndexedTriangleStripSetType		indexedTriangleStripSet;
	INVLineSetType				lineSet;
	INVNonIndexedShapeType			nonIndexedShape;
	INVNurbsCurveType			nurbsCurve;
	INVNurbsSurfaceType			nurbsSurface;
	INVPointSetType				pointSet;
	INVQuadMeshType				quadMesh;
	INVShapeType				shape;
	INVSphereType				sphere;
	INVText2Type				text2;
	INVText3Type				text3;
	INVTriangleStripSetType			triangleStripSet;

	/*
	!	Property Node Classes
	*/

	INVBaseColorType			baseColor;
	INVComplexityType			complexity;
	INVCoordinate3Type			coordinate3;
	INVCoordinate4Type			coordinate4;
	INVDrawStyleType			drawStyle;
	INVEnvironmentType			environment;
	INVFontType				font;
	INVInfoType				info;
	INVLabelType				label;
	INVLightModelType			lightModel;
	INVLinearProfileType			linearProfile;
	INVMaterialType				material;
	INVMaterialBindingType			materialBinding;
	INVMatrixTransformType			matrixTransform;
	INVNormalType				normal;
	INVNormalBindingType			normalBinding;
	INVNurbsProfileType			nurbsProfile;
	INVPackedColorType			packedColor;
	INVPickStyleType			pickStyle;
	INVProfileCoordinate2Type		profileCoordinate2;
	INVProfileCoordinate3Type		profileCoordinate3;
	INVResetTransformType			resetTransform;
	INVRotationType				rotation;
	INVRotationXYZ				rotationXYZ;
	INVScaleType				scale;
	INVShapeHintsType			shapeHints;
	INVTexture2Type				texture2;
	INVTexture2TransformType		texture2transform;
	INVTextureCoordinate2Type		textureCoord2;
	INVTextureCoordinateBindingType		textureCoordBinding;
	INVTextureCoordinateCubeType		textureCoordCube;
	INVTextureCoordinateCylinderType	textureCoordCylinder;
	INVTextureCoordinateEnvironmentType	textureCoordEnvironment;
	INVTextureCoordinateFunctionType	textureCoordFunction;
	INVTextureCoordinatePlaneType		textureCoordPlane;
	INVTextureCoordinateSphereType		textureCoordSphere;
	INVTransformType			transform;
	INVTranslationType			translation;
	INVUnitsType				units;

	/*
	!	Group Node Classes
	*/

	INVArrayType				array;
	INVCustomNodeType			customNode;
	INVFileType				file;
	INVGroupType				group;
	INVLayerGroupType			layerGroup;
	INVMultipleCopyType			multipleCopy;
	INVPathSwitchType			pathSwitch;
	INVSelectionType			selection;
	INVSeparatorType			separator;

	/*
	!	Light Node Classes
	*/

	INVDirectionalLightType			directionalLight;
	INVLightType				light;
	INVPointLightType			pointLight;
	INVSpotLightType			spotLight;

	/*
	!	Camera Node Classes
	*/

	INVCameraType				camera;
	INVOrthographicCamera			orthographicCamera;
	INVPerspectiveCamera			perspectiveCamera;
	} INVNodeUnion;

typedef struct
	{
	int		id;
	INVNodeUnion	NodeUnion;
	INVMaterialType	mat;
	INVObject	ShapeCache;
	LLlist		ChildList;
	int		specified[MAX_SYMBOLS];
	GLuint		MaterialDL;
	GLuint		TextureObject;
	int		count, LineNum;
	int		*PolyPicks, PolyCount, TexBoxFound;
	int		TexBoxMedium, TexBoxLarge;
	float		TexLargeDist, TexMediumDist, TexBoxRatio;
	Point3D		min, max;
	char		*name;
	} INVSceneNode;

#define	INV_MAX_STRIP_LENGTH	14

typedef struct
	{
	int		VertexCount;
	int		NormalCount;
	int		TexCoordCount;
	int		TriCount;
	int		StripCount;
	int		StripLengths[INV_MAX_STRIP_LENGTH + 1];
	int		MaterialSwitchCount;
	int		TextureSwitchCount;
	int		MatrixPushes;
	int		MatrixPops;
	int		MatrixLoads;
	int		MatrixMults;
	int		TexMatrixPushes;
	int		TexMatrixPops;
	int		TexEnvCounts;
	int		TextureMemory;
	} INVPerformance;

typedef struct
	{
	Point3D	min, max;
	} INVExtentsType;

enum
	{
	TEXGEN_NONE, TEXGEN_PLANAR, TEXGEN_SPHERE
	};

typedef struct
	{
	/*
	!	material contexts
	*/
	GLUMaterialType	material;
	GLuint		MaterialDL;

	/*
	!	texture contexts
	*/
	GLuint		TextureObject;
	int		TexturePushed;
	int		TexgenMode;
	Point3D		Snormal, Tnormal;
	} INVContext;

typedef struct
	{
	char	*filename;
	GLuint	TextureObject;
	} INVTextureCache;

typedef struct
	{
	char		*name;
	INVSceneNode	*node;
	} INVDefine;

typedef struct
	{
	int		ShapeType;
	INVSceneNode	*ShapeNode, *C3Node, *NormalNode, *SepNode;
	int		PolyIndex;
	long		*pptr;
	} INVPolyPickType;

/*--SHAPE NODE ENUMERATIONS--------------------------------------------------*/

enum TOKENS
        {
        /*
        !       Shape Node Classes
        */

        ConeSym, CubeSym, CylinderSym, FaceSetSym, IndexedFaceSetSym,
        IndexedLineSetSym, IndexedNurbsCurveSym, IndexedNurbsSurfaceSym,
        IndexedShapeSym, IndexedTriangleStripSetSym, LineSetSym,
        NonIndexedShapeSym, NurbsCurveSym, NurbsSurfaceSym, PointSetSym,
        QuadMeshSym, ShapeSym, SphereSym, Text2Sym, Text3Sym,
        TriangleStripSetSym, VertexShapeSym,

        /*
        !       Property Node Classes
        */

        BaseColorSym, ComplexitySym, Coordinate3Sym, Coordinate4Sym,
        DrawStyleSym, EnvironmentSym, FontSym, InfoSym, LabelSym,
        LightModelSym, LinearProfileSym, MaterialSym, MaterialBindingSym,
        MatrixTransformSym, NormalSym, NormalBindingSym, NurbsProfileSym,
        PackedColorSym, PickStyleSym, ProfileSym, ProfileCoordinate2Sym,
        ProfileCoordinate3Sym, ResetTransformSym, RotationSym, RotationXYZSym,
        ScaleSym, ShapeHintsSym, Texture2Sym, Texture2TransformSym,
        TextureCoordinate2Sym, TextureCoordinateBindingSym,
        TextureCoordinateCubeSym, TextureCoordinateCylinderSym,
        TextureCoordinateEnvironmentSym, TextureCoordinateFunctionSym,
        TextureCoordinatePlaneSym, TextureCoordinateSphereSym, TransformSym,
        TranslationSym, UnitsSym,
        /*
        !       Group Node Classes
        */

        ArraySym, CustomNodeSym, FileSym, GroupSym, LayerGroupSym,
        MultipleCopySym, PathSwitchSym, SelectionSym, SeparatorSym, SwitchSym,

        /*
        !       Light Node Classes
        */

        DirectionalLightSym, LightSym, PointLightSym, SpotLightSym,

	/*
	!	Define Stuff
	*/

	DEFSym, USESym, 

        /*
        !       Camera Node Classes
        */

        CameraSym, OrthographicCameraSym, PerspectiveCameraSym,

        /*
        !       Field Tokens
        */

        angleSym, repeatIntervalSym, blendColorSym, translationSym,
        centerSym, normalSym, vertexOrderingSym, faceTypeSym, shapeTypeSym,

        numElements1Sym, numElements2Sym, numElements3Sym, separation1Sym,
        separation2Sym, separation3Sym, originSym, rgbSym, typeSym, valueSym,
        partsSym, bottomRadiusSym, heightSym, pointSym, widthSym, depthSym,
        classNameSym, fieldsSym, customDataSym, radiusSym, intensitySym,
        colorSym, directionSym, styleSym, lineWidthSym, linePatternSym,
        ambientIntensitySym, ambientColorSym, attenuationSym, fogTypeSym,
        fogComputedSym, fogColorSym, fogDensitySym, fogNearDistanceSym,
        fogFarDistanceSym, startIndexSym, numVerticesSym, nameSym, sizeSym,
        coordIndexSym, materialIndexSym, normalIndexSym, textureCoordIndexSym,
        knotVectorSym, numUControlPointsSym, numVControlPointsSym,
        numSControlPointsSym, numTControlPointsSym, uKnotVectorSym,
        vKnotVectorSym, sKnotVectorSym, tKnotVectorSym, stringSym, labelSym,

        modelSym, nearColorSym, farColorSym, indexSym, linkageSym,
        diffuseColorSym, specularColorSym, emissiveColorSym, shininessSym,
        transparencySym, matrixSym, vectorSym, numControlPointsSym,
        viewportMappingSym, positionSym, orientationSym, aspectRatioSym,
        nearDistanceSym, farDistanceSym, focalDistanceSym, rgbaSym, pathSym,
        heightAngleSym, locationSym, verticesPerColumnSym, numPointsSym,
        verticesPerRowSym, whatToResetSym, rotationSym, axisSym,
        scaleFactorSym, hintsSym, creaseAngleSym, dropOffRateSym,
        cutOffAngleSym, whichChildSym, spacingSym, justificationSym,
        componentSym, filenameSym, minFilterSym, magFilterSym, wrapSSym,
        wrapTSym, coordSym, distanceFromOriginSym, scaleOrientationSym,
        unitssym, directionSSym, directionTSym,

        INV_FIRST, INV_CENTER, INV_LAST, INV_OBJECT_SPACE, INV_SCREEN_SPACE,
        INV_BOUNDING_BOX, INV_FILLED, INV_LINES, INV_POINTS, INV_INVISIBLE,
        INV_NONE, INV_LINEAR, INV_EXPONENTIAL, INV_EXPONENTIAL_SQUARED,
        INV_PER_VERTEX, INV_PER_PIXEL, INV_BASE_COLOR, INV_PHONG, INV_DEPTH,
        INV_START_FIRST, INV_START_NEW, INV_ADD_TO_CURRENT, INV_DEFAULT,
        INV_OVERALL, INV_PER_PART, INV_PER_FACE, INV_PER_FACE_INDEXED,
        INV_VERTEX_INDEXED, INV_ADJUST_CAMERA,
        INV_CROP_VIEWPORT_FILL_FRAME, INV_CROP_VIEWPORT_LINE_FRAME,
        INV_CROP_VIEWPORT_NO_FRAME, INV_LEAVE_ALONE, INV_PICKABLE,
        INV_UNPICKABLE, INV_TRANSFORM, INV_BBOX, INV_X, INV_Y, INV_Z,
        INV_SOLID, INV_ORDERED, INV_CONVEX, INV_SURFACE, INV_UNORDERED,
        INV_CONCAVE, INV_LEFT, INV_RIGHT,
        INV_INTENSITY, INV_TRANSPARENCY, INV_POINT, INV_BILINEAR,
        INV_MIPMAP_POINT, INV_MIPMAP_LINEAR, INV_MIPMAP_BILINEAR,
        INV_MIPAMP_TRILINEAR, INV_PER_VERTEX_INDEXED, INV_S, INV_T,
        INV_METERS, INV_CENTIMETERS, INV_MILLIMETERS, INV_MICROMETERS,
        INV_MICRONS, INV_NANOMETERS, INV_ANGSTROMS, INV_KILOMETERS,
        INV_FEET, INV_INCHES, INV_YARDS, INV_MILES, INV_NAUTICAL_MILES,
        INV_REPEAT, INV_CLAMP, INV_MODULATE, INV_DECAL, INV_BLEND,

        INV_COUNTERCLOCKWISE, INV_CLOCKWISE, INV_UNKNOWN_FACE_TYPE,

        INV_NUM_SYMBOLS,

        INVFilenameSym, INVId, INVLiteral, INVLParen, INVRParen, INVLCurly, 
	INVRCurly, INVLBrace, INVRBrace, INVTildaSym, INVSemiColon, INVComma, 
	INVQuote, INVAssignOp, INVPipeSym, INVEofSym, INVNull, INVErrorChar, 
	INVUnassignedToken, INV_NUM_TOKENS
        };
#endif

/*--GLOBAL IMPORTS-----------------------------------------------------------*/

#ifdef LIBRARY_CODE

#ifndef PARSER
extern	jmp_buf			INVExceptionBuf;
#endif

#ifndef MY_INVPARSER_CONTEXT
extern	void			PushContext(void);
extern	void			PopContext(void);
extern	void			PushTexture(void);
extern	void			PopTexture(void);
extern  void    		InitContext(void);
extern	void			CloseContext(void);

extern	INVContext		INVCurrentContext;
extern	OpenGLStateType		INVOpenGLState;

#endif

#ifndef INV_MATERIALS
extern	INVMaterialType		INVDefaultMaterial;
#endif

#ifndef UTILITY
extern	void	InvResetMalloc(void);
extern	void	*InvMallocNoRec(size_t);
extern	void	*InvMalloc(size_t);
extern	char	*InvStrdup(char *);
extern	char	*InvLastFilename(char *);
#endif

#ifndef SCANNER
extern	void	INVReportParseError(const char *, ...);
extern	int	INVInitIO(char *);
extern	void	INVCloseIO(void);
extern	void	INVMatch(int);

extern	int	INVCurrentToken;
extern	char	INVTokenBuffer[MAX_STR_LEN];
#endif

#ifndef MATRIX_CODE
extern	void	INVLoadId(INVMatrixType);
extern	void	INVScale(INVMatrixType, float, float, float);
extern	void	INVTranslate(INVMatrixType, float, float, float);
extern	void    INVRot(INVMatrixType, float, char);
#endif

#ifndef TEXTURE
extern	int	INVTextureFlag;
#endif

#else

extern	LLlist		INVParseInventorFile(char *);
extern	int			INVWriteAsDXF(LLlist, char *);
extern	GLuint		OIVCreateDisplayList(char *, bool bInitMaterial = true);
extern	GLuint		OIVRealInventor(char *);
extern	void		OIVInitMaterial(bool bFlag);
extern	void		OIVCleanUp(void);
extern	void		OIVGammaCorrection(float fGammaFactor = 1.0f);
#ifndef __sgi
extern	void		OIVSetTexturesPath(char* pTexturesPath);
#endif // not __sgi
extern	void		INVGetSceneLimits(INVPoint3D, INVPoint3D);
extern	void		INVEnableTextures(int);
extern	void		INVPrintPerformanceData(void);
extern	int			INVValidFile(char *);

extern	size_t		INVSpaceUsed;
#endif


/*--END FILE--(invparse.h)---------------------------------------------------*/
