//		   Name: MfcUtils.h
//
//	Description: Header file for MFC Utilities
//
//		Version		Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		1.00		4/29/99		Weaver		Creation
//
//
//	Contents:
//				1	Gloobal functions
//					a)	int		WinUMsgBox(char* pCaption, UINT nType, PSTR sz,...);
//					b)	int		WinUMsgBox(char* pCaption, ...);
//					c)	string	FontResourceToFile( UINT nID);
//					d)	void	StatusBarMessage(char* fmt, ...);
//					e)	bool	CopyGLFrontToClipboard(CWnd* pWnd, int nWidth, int nHeight);
//					f)	void	CopyGLFrontToClipboard(COleDataSource * pSource, int nWidth, int nHeight); 
//					g)	void	CopyGLFrontToClipboardAsEnhancedMetafile(COleDataSource * pSource, int nWidth, int nHeight); 
//					h)	string	GetCurrentMessage( );
//					i)  void	SaveGLToRAW(std::string strFilename);
//					j)	void	SaveGLToBMP(std::string strFilename);
//					k)	void	SaveGLToTGA(std::string strFilename);
//				2	CDirectoryBrowse Class
//				3	CResolutionChanger Class
//				4	CWinUMenu	Class
//				5	CConsoleWindow Class
//				6	CProgressBar
//				7	CWinUSplitterWnd Classq
//				8	CVTBDialog
//				9	CRegistry Classes


#ifdef _DEBUG
#pragma comment(lib, "MfcUtils_v1.0d.lib")
#else
#pragma comment(lib, "MfcUtils_v1.0.lib")
#endif

#pragma once
#include "gl\gl.h"  
#include "gl\glu.h" 
#include "gl\glaux.h"
#include <string>
#include <vector>
#include <afxcmn.h>
#include <afxext.h>
#include <afxole.h>


//		Gobal Functions																								1

//		   Name: int WinUMsgBox(char* pCaption, UINT nType, PSTR sz,...);
//
//	Description: Formatted message box
//
//		Version		Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		1.00		4/28/99		Weaver		Creation

int WinUMsgBox(char* pCaption, UINT nType, PSTR sz,...);

//		   Name: int WinUMsgBox(char* pCaption, UINT nType, PSTR sz,...);
//
//	Description: Formatted message box
//
//		Version		Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		1.00		4/29/99		King		Creation

int WinUMsgBox(char* pCaption, ...);


//		   Name: FontResourceToFile( UINT nID);
//
//	Description: Retreives a font resource to a temp file
//
//		Version		Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		1.00		4/29/99		Hardwick	Creation

std::string FontResourceToFile( UINT nID);



//		   Name: StatusBarMessage(char* fmt, ...);
//
//	Description: Changes status bar message like sprintf
//
//		Version		Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		1.00		5/03/99		Web

void StatusBarMessage(char* fmt, ...);



//		   Name: CopyGLFrontToClipboard(CWnd* pWnd, int nWidth, int nHeight);
//
//	Description: Copy Gl Window to clipboard
//
//
//	Usage Example:
//		void CMeterView::OnEditCopy() 
//		{
//			BeginGL();
//			if( !CopyGLFrontToClipboard(this, m_nWidth, m_nHeight) )
//				AfxMessageBox("Copy failed");
//			EndGL();
//		}
//
//		Version		Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		1.00		5/04/99		Weaver		Creation
bool	CopyGLFrontToClipboard(CWnd* pWnd, int nWidth, int nHeight);
void	CopyGLFrontToClipboard(COleDataSource * pSource, int nWidth, int nHeight); 
void	CopyGLFrontToClipboardAsEnhancedMetafile(COleDataSource * pSource, int nWidth, int nHeight); //crashes when pasting EMF??????

//		   Name: string GetCurrentMessage( );
//
//	Description: Returns the error message
//
//
//	Usage Example:
//			m_glstrInfo.SetFont("crap.txt");
//			MsgBox( "The error was %d", GetLastError() );
//			MsgBox( (char*)GetCurrentMessage( ).c_str() );
//
//		Version		Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		1.00		5/04/99		Weaver		Creation

std::string GetCurrentMessage( );



void	SaveGLToRAW(std::string strFilename);
void	SaveGLToBMP(std::string strFilename);
void	SaveGLToTGA(std::string strFilename);
void	SaveGLToBMP32(std::string strFile);



//		   Name: CDirectoryBrowse Class																				2						
//		
//	Description: Formatted message box
//
//		Version		Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		1.00		4/29/99		Weaver		From Web
#if !defined(MFC_UTILS_CDIRECTORYBROWSE)
#define MFC_UTILS_CDIRECTORYBROWSE
class CDirectoryBrowse  
{
public:
	void GetDirectory(CString &directory);	// returns selected directory
	bool DoModal();							// invokes dialog.  returns
											// TRUE if OK selected
	void SetTitle(char *title);				// sets title text
	CDirectoryBrowse();
	virtual ~CDirectoryBrowse();

protected:
	CString m_title;
	CString m_directory;
};
#endif








//		   Name: CResolutionChanger Class																			3
//
//	Description:	Reads the resolutions that you video card supports
//					Allows you to retrieve the resolution as a formatted string
//
//					Note: put ChangeDisplaySettings(NULL,0); in you Apps ExitInstance()
//						to restore the original resolution or call RestoreResolution()
//
//		Version 	Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		01.01.00	03/03/99	Weaver		Creation
#if !defined(MFC_UTILS_CRESOLUTIONCHANGER)
#define MFC_UTILS_CRESOLUTIONCHANGER

class CResolutionChanger
{
	struct RESOLUTION
	{
		std::string strName;
		INT32		nWidth;
		INT32		nHeight;
		INT32		nBitDepth;
	};
private:	//Private Member Variables
	std::vector<RESOLUTION> m_vResolutions;

private:	//Private Member Functions
	void	ListDisplaySettings(void);

public:		//Public Member Functions
	CResolutionChanger();
	BOOL		ChangeResolution(INT32 w, INT32 h, INT32 bitdepth);
	void		RestoreResoltion(void);
	int			GetNumResolutions();
	std::string	GetResolution(int i);
	void		GetResolutionValues( int i, INT32 &nWidth, INT32 &nHeight, INT32 &nBitDepth);
	void		SetResolution( int i);
};

#endif




//		   Name: CWinUMenu																							4
//
//	Description:  Create Popup menu.
/*

	Receive messages through:

  	afx_msg void	OnMenu( UINT );
	
	ON_COMMAND_RANGE( ID_WINU_MENU, ID_WINU_MENULAST,  OnMenu)
	
	void <My CWnd derived class>::OnMenu( UINT nID) 
	{
		NOTE:  Selected item is nID - ID_WINU_MENU
	}
*/
//
//		Version 	Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		01.01.00	04/08/99	Weaver		Creation
#if !defined(MFC_UTILS_CWINUMENU)
#define MFC_UTILS_CWINUMENU

#define		ID_WINU_MENU			50000
#define		ID_WINU_MENULAST		51000

class CWinUMenu : public CMenu
{
	
public: 
	CWinUMenu(CWnd* pParent);
	~CWinUMenu();
	void	Reset();
	void	Add( char** pEntries, UINT nID = 0);
	void	Show( CPoint cPoint);

private:
	CMenu*				AddItem( CMenu* pMenu, std::string strItem );
	CWnd*				m_pParent;
	UINT				m_nID;
	std::vector<CMenu*>	m_vMenuPopups;
	std::vector<CMenu*>	m_vParents;
	UINT				m_nIndex;
};
#endif







//		   Name: CConsoleWindow Class																				5
//
//	Description:	Creates a console window if the app was called with -D
//			
//	Use:
//					Create an instance as a member of your CWinApp derived class
//					Call InitConsole() in InitInstance   (it will switch on the -D flag)
//
//		Version 	Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		01.01.00	04/30/99	Hardwick	Creation

#if !defined(MFC_UTILS_CCONSOLEWINDOW)
#define MFC_UTILS_CCONSOLEWINDOW

class CConsoleWindow
{
private:
	PROCESS_INFORMATION    pi;
	bool m_bState;
public:
	int InitConsoleWindow( );
	CConsoleWindow();
	~CConsoleWindow();
	void Kill();
	void Start();
	void Toggle();

};

#endif
		
// ProgressBar.h : header file																						6
//
// Drop-in status bar progress control
//
// Written by Chris Maunder (chrismaunder@codeguru.com)
// Copyright (c) 1998.
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed unmodified by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name is included. If 
// the source code in this file is used in any commercial application 
// then an email to me would be nice.
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability if it causes any damage to your
// computer, causes your pet cat to fall ill, increases baldness or
// makes you car start emitting strange noises when you start it up.
//
// Expect bugs.
// 
// Please use and enjoy. Please let me know of any bugs/mods/improvements 
// that you have found/implemented and I will fix/incorporate them into this
// file. 
//
// Version 1.1  Chris Maunder (chrismaunder@codeguru.com)
//
// Version 1.2  Michael Martin 07 Aug 1998	mmartin@netspace.net.au
//              Added code to enable progress bar to appear in any pane
//              of the status bar.
//
//              Chris Maunder 17 Aug 1998 
//              Fixed Pane text restoration, and general cleanup.
//
/////////////////////////////////////////////////////////////////////////////

#ifndef _INCLUDE_PROGRESSBAR_H_
#define _INCLUDE_PROGRESSBAR_H_


/////////////////////////////////////////////////////////////////////////////
// CProgressBar -  status bar progress control
//
// Copyright (c) Chris Maunder, 1997
// Please feel free to use and distribute.

class CProgressBar: public CProgressCtrl
// Creates a ProgressBar in the status bar
{
public:
	CProgressBar();
	CProgressBar(LPCTSTR strMessage, int nSize=100, int MaxValue=100, 
                 BOOL bSmooth=FALSE, int nPane=0);
	~CProgressBar();
	BOOL Create(LPCTSTR strMessage, int nSize=100, int MaxValue=100, 
                BOOL bSmooth=FALSE, int nPane=0);

	DECLARE_DYNCREATE(CProgressBar)

// operations
public:
	BOOL SetRange(int nLower, int nUpper, int nStep = 1);
	BOOL SetText(LPCTSTR strMessage);
	BOOL SetSize(int nSize);
	COLORREF SetBarColour(COLORREF clrBar);
	COLORREF SetBkColour(COLORREF clrBk);
	int  SetPos(int nPos);
	int  OffsetPos(int nPos);
	int  SetStep(int nStep);
	int  StepIt();
	void Clear();

// Overrides
	//{{AFX_VIRTUAL(CProgressBar)
	//}}AFX_VIRTUAL

// implementation
protected:
	int		m_nSize;		// Percentage size of control
	int		m_nPane;		// ID of status bar pane progress bar is to appear in
	CString	m_strMessage;	// Message to display to left of control
    CString m_strPrevText;  // Previous text in status bar
	CRect	m_Rect;			// Dimensions of the whole thing

	CStatusBar *GetStatusBar();
	BOOL Resize();

// Generated message map functions
protected:
	//{{AFX_MSG(CProgressBar)
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

};

#endif
/////////////////////////////////////////////////////////////////////////////


//		   Name: CWinUSplitterWnd Class																				7
//
//	Description:	Splitter Window to allow for various fixed sized panes
//			
//	Use:
//					Look at comments in declarations for locations to put functuions
//
//		Version 	Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		01.01.00	04/30/99	Weaver	Creation

#if !defined(MFC_UTILS_CWINUSPLITTERWND)
#define MFC_UTILS_CWINUSPLITTERWND

class CWinUSplitterWnd : public CSplitterWnd
{
	bool m_bCreated;
	int m_nRows;
	int m_nCols;
	struct PANE
	{
		int nHeight;
		int nWidth;
		CRuntimeClass* pClass;
	};
	std::vector<std::vector<PANE>*> m_vPanes;
	void	RecalcLayout( bool bFlag=false )		{ if( bFlag) CSplitterWnd::RecalcLayout(); }
public:
	bool	SetSize( int nRows, int nCols );	//1st thing called on childframe's OnCreateClient
	bool	SetPane( int nRow, int nCol, CRuntimeClass* pClass, int nWidth = -1, int nHeight = -1); //2nd thing called on childframe's OnCreateClient
	CWinUSplitterWnd() ;
	~CWinUSplitterWnd();
	bool	CreateViews( CWnd* pWnd, CCreateContext* pContext);//3rd thing called on childframe's OnCreateClient
	void	HandleOnSize( int cx, int cy ); //childframe's OnSize
protected:
	afx_msg void OnMouseMove(UINT nFlags, CPoint point) { }
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point) { }

	 DECLARE_MESSAGE_MAP()
};

#endif










//		   Name: CVTBDialog																							8
//
//	Description:	Dialog Class for Modal and Modeless Dialogs
//			
//	Use:
//					
//
//		Version 	Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		01.01.00	05/21/99	Weaver	Creation

#if !defined(MFC_UTILS_CVTBDIALOG)
#define MFC_UTILS_CVTBDIALOG


 
class CWinUDialog : public CDialog
{
private:
	bool m_bCreated;
	CWnd* m_pParent;
public:
	CWinUDialog( UINT nIDTemplate, CWnd* pParentWnd=NULL  ) 
		: CDialog( nIDTemplate, pParentWnd ) 
	{
		m_bCreated = false; 
		m_pParent = NULL;
	}
	
	void NotifyParent(UINT nCode) 
	{ 
		if( m_pParent != NULL)
		{
			NMHDR cHdr;
			cHdr.hwndFrom = m_hWnd;
			cHdr.code = nCode;
			m_pParent->SendMessage(WM_NOTIFY,(WPARAM)nCode,(LPARAM)&cHdr);
		}
	}
	

	int	Popup( CWnd* pParent=NULL,UINT uiID = 1 )
	{
		if( m_pParent == NULL && pParent != NULL)
			m_pParent = pParent;
		if( uiID == 1)
			return DoModal();
		else
		{
			if( !m_bCreated )
			{
				Create(uiID);
				m_bCreated = true;
			}
			ShowWindow(SW_SHOW);
		}
		return 1;
	}
	bool IsShown() 
	{ 
		if( ::IsWindow(m_hWnd)) 
			return ((GetStyle() & WS_VISIBLE)!=0); 
		else 
			return false;
	}


};


#endif



//#if !defined(MFC_UTILS_CREGISTRY)
//#define MFC_UTILS_CREGISTRY


/////////////////////////////////////////////////////////////////////////////										9
// Copyright (C) 1997 by Joerg Koenig
// All rights reserved
//
// Distribute freely, except: don't remove my name from the source or
// documentation (don't take credit for my work), mark your changes (don't
// get me blamed for your possible bugs), don't alter or remove this
// notice.
// No warrantee of any kind, express or implied, is included with this
// software; use at your own risk, responsibility for damages (if any) to
// anyone resulting from the use of this software rests entirely with the
// user.
//
// Send bug reports, bug fixes, enhancements, requests, flames, etc., and
// I'll try to keep a version up to date.  I can be reached as follows:
//    J.Koenig@adg.de                 (company site)
//    Joerg.Koenig@rhein-neckar.de    (private site)
/////////////////////////////////////////////////////////////////////////////

// last revised $Date: 2000/10/24 18:40:25 $ $Revision: 1.1.1.1 $

// -----------------------------------------------------------------------------
// Description:
//	This headerfile describes the following classes:
//		CRegistry	registry wrapper (non-abstract base class)
//		CRegVal		registry information holder
//
//	Inheritance is neccessary only if you want to use the "RegistryTreeWalk()"
//	function declared in CRegistry. Then you have to derive your own class and to
//	override the "OnValueHit()" and/or the "OnKeyHit()" virtual member functions
//	(see below).
// -----------------------------------------------------------------------------


#ifdef _EXPLICIT
	#pragma message (__FILE__ ": Redefining macro _EXPLICIT")
	#undef _EXPLICIT
#endif

#if _MSC_VER >= 1100
	#define _EXPLICIT	explicit
#else
	#define _EXPLICIT
#endif

#if defined(__AFXWIN_H__) && ! defined(_REG_MFC_BUILD) && ! defined(_REG_NO_MFC_BUILD)
	// assume that this is an MFC build
	#define _REG_MFC_BUILD
#endif

#ifndef STRICT
	#pragma message (__FILE__ ": You should #define STRICT to keep CRegistry behave correct in all circumstances.")
#endif

#include <winreg.h>

class CRegVal ;


class CRegistry {
	HKEY		m_hCurrentKey ;

	public:

#ifndef _REG_NO_TREEWALK
		enum {
			// the following constants are for the RegistryTreeWalk()
			// member function:
			RTW_NotifyValue		= 1,
			RTW_NotifyKey		= 2,
			RTW_NotifyAll		= 3		// same as "RTW_NotifyValue | RTW_NotifyKey"
		};
#endif	// _REG_NO_TREEWALK

	public:
		_EXPLICIT CRegistry( HKEY OpenKey = HKEY_CURRENT_USER ) ;
		CRegistry( const CRegistry & ) ;
		virtual ~CRegistry() ;

	public:		// attributes
		// GetKey() returns the current HKEY
		HKEY		GetKey() const ;

		// Change the current key of the registry and return
		// the old one.
		HKEY		SetKey( HKEY ) ;

	public:		// operations
		// "Connect()" connects the registry object with either
		// the local or a remote computer. A connection to the
		// local computer will be forced by setting the <Remote>
		// parameter to NULL.
		// if the function fails (i.e. returns FALSE), then
		// the previously connected HKEY will be restored, so
		// there is no need to reset it (via the SetKey() member
		// function).
		// The <Remote> string shall be in UNC notation (such as
		// "\\computer" - note that you have to write each backslash
		// twice in C/C++ code) !
		//
		// NOTE: You cannot create subkeys in the root of a
		//		 predefined key if you've called "Connect()".
		//		 Thus the following code will *not* work:
		//			reg.Connect( TEXT("\\\\picard") ) ;
		//			reg.SaveKey( TEXT("MyOwnKeyInTheRoot"), TEXT("AValue"), regval ) ;
		//		 if "MyOwnKeyInTheRoot" does not already exist.
		//
		// NOTE: that you can actually connect to either
		//		 HKEY_LOCAL_MACHINE or HKEY_USERS only. This is a
		//		 restriction by Windows ...
		BOOL		Connect(
						LPCTSTR Remote,
						HKEY = HKEY_LOCAL_MACHINE
					) ;

		// Load the value <ValName> of a given subkey <SubKey>
		// of the current key in the registry. The value's data will
		// be stored in <Data>. Returns TRUE on success, FALSE on
		// failure (i.e. <SubKey> or <ValName> (or both) not found,
		// or not enough permissions)
		BOOL		LoadKey(		// load data of any type
						LPCTSTR SubKey,
						LPCTSTR ValName,
						CRegVal & Data
					) const;
		BOOL		LoadKey(		// load a number
						LPCTSTR SubKey,
						LPCTSTR ValName,
						DWORD & Number
					) const;
		BOOL		LoadKey(		// load binary data
						LPCTSTR SubKey,
						LPCTSTR ValName,
						BYTE * pBuffer,
						DWORD dwSize
					) const;

#ifdef _REG_MFC_BUILD
		BOOL		LoadKey(		// load a string
						LPCTSTR SubKey,
						LPCTSTR ValName,
						CString & String
					) const;
		BOOL		LoadKey(		// load an array of strings
						LPCTSTR SubKey,
						LPCTSTR ValName,
						CStringArray & arrString
					) const;
#endif	// _REG_MFC_BUILD

		// save <Data> as <ValName> in the given <SubKey> of the
		// current key in the registry. If <ValName> does not exist,
		// it will be created, elsewise its value will be changed.
		// if <SubKey> does not exist, then it will be created too
		// (with all sub-subkeys in it).
		// You have to separate subkeys like a DOS-Path
		// (example: "ADG\\Pegasus\\General\\Protocol" is a valid subkey)
		// Returns TRUE on success or FALSE on failure (i.e. not enough
		// permissions)
		BOOL		SaveKey(		// store data of any type
						LPCTSTR SubKey,
						LPCTSTR ValName,
						const CRegVal & Data
					) const;
		BOOL		SaveKey(		// store a number
						LPCTSTR SubKey,
						LPCTSTR ValName,
						const DWORD & Number
					) const;
		BOOL		SaveKey(		// store binary data
						LPCTSTR SubKey,
						LPCTSTR ValName,
						const LPBYTE pBuffer,
						const DWORD dwSize
					) const;
		BOOL		SaveKey(		// store a string
						LPCTSTR SubKey,
						LPCTSTR ValName,
						LPCTSTR String
					) const;

#ifdef _REG_MFC_BUILD
		BOOL		SaveKey(	// store an array of strings
						LPCTSTR SubKey,
						LPCTSTR ValName,
						const CStringArray & arrString
					) const;
#endif	// _REG_MFC_BUILD

		// "DeleteKey()" deletes the given Key in the registry. If
		// <ValName> is equal to NULL, then the complete key will
		// be removed (with all of its subkeys).
		// Otherwise only the given <ValName> of that
		// <SubKey> is affected.
		BOOL		DeleteKey(
						LPCTSTR SubKey,
						LPCTSTR ValName = 0
					) const ;

		// The "HasEntries()" member function checks wether the
		// given <SubKey> of the current key has other subkeys
		// and/or values. If so - it returns TRUE, otherwise FALSE.
		// If you define a buffer for either <SubKeyCount> or
		// <ValueCount> (or both), the number of hits of that type of
		// entry will be stored in it.
		BOOL		HasEntries(
						LPCTSTR SubKey,
						DWORD * SubKeyCount = 0,
						DWORD * ValueCount = 0
					) const ;

#ifndef _REG_NO_TREEWALK
		// The "RegistryTreeWalk()" function walks through the registry
		// beginning at the current key and the given <SubKey>. if
		// <SubKey> is NULL, then the walk begins at root level.
		// For every value the function "OnValueHit()" will be called.
		// Note that OnValueHit() and OnKeyHit() are dummies by default
		// (don't made it pure, because usage of the RegistryTreeWalk()
		// function may be very seldom...).
		// You have to derive your own class and to override
		// these handler function(s), if you want to use the
		// "RegistryTreeWalk()" function!
		// NOTE: If you specify a <SubKey> other than NULL, the "OnKeyHit()"
		// notification-handler for this initial key will NOT be called !
		// "RegistryTreeWalk()" returns TRUE on success or FALSE on failure
		// (i.e. failure on accessing the registry (watch your DEBUG-Window
		// in debugging mode) or one of the notification-handlers
		// (OnValueHit(), OnKeyHit()) forces the walk to abort)
		BOOL		RegistryTreeWalk(
						LPCTSTR SubKey = 0,
						UINT Flags = CRegistry::RTW_NotifyAll
					);

	public:		// overridables
		// The "OnValueHit()" function will be called from within the
		// "RegistryTreeWalk()" function, whenever a valid value is
		// hit. You should return TRUE, if you want to continue the
		// walk or FALSE to abort. This function will only be called
		// if you specified either "RTW_NotifyValue" or "RTW_NotifyAll"
		// in the flags to "RegistryTreeWalk()".
		// The <Depth> parameter indicates the depth of the walk. It
		// will be zero on root-level (or the given <SubKey> to
		// "RegistryTreeWalk()", see above), one on the subkey, two on
		// the sub-subkey and so on.
		// NOTE that the default implementation of this function returns
		// FALSE always.
		virtual BOOL	OnValueHit(
							LPCTSTR Key,
							LPCTSTR ValueName,
							const CRegVal & Value,
							USHORT Depth
						);

		// "OnKeyHit()" will be called from the "RegistryTreeWalk()" function
		// only if you have specified either "RTW_NotifyKey" or "RTW_NotifyAll"
		// in the flags to "RegistryTreeWalk()".
		// For the <Depth> parameter see "OnValueHit()" above.
		// NOTE that the default-implementation returns always FALSE !
		virtual BOOL	OnKeyHit( LPCTSTR Key, USHORT Depth ) ;
#endif	// _REG_NO_TREEWALK


	public:		// static members ...
		// same as above, but at static scope for lower-level usage.
		static BOOL		LoadKey(
							LPCTSTR SubKey,
							LPCTSTR ValName,
							CRegVal & Data,
							HKEY Key
						) ;

		// same as above, but at static scope for lower-level usage.
		static BOOL		SaveKey(
							LPCTSTR SubKey,
							LPCTSTR ValName,
							const CRegVal & Data,
							HKEY Key
						) ;

		// same as above, but at static scope for lower-level usage.
		static BOOL		DeleteKey(
							LPCTSTR SubKey,
							LPCTSTR ValName,
							HKEY Key
						) ;

		// same as above, but at static scope for lower-level usage.
		static BOOL		HasEntries(
							LPCTSTR SubKey,
							DWORD * SubKeyCount,
							DWORD * ValueCount,
							HKEY Key
						) ;

	private:
#ifndef _REG_NO_TREEWALK
		// The "RegTreeWalk()" will be called from "RegistryTreeWalk()" or from
		// itself recursivly.
		BOOL			RegTreeWalk( LPCTSTR SubKey, HKEY hSubKey, UINT Flags, USHORT Depth ) ;
#endif	// _REG_NO_TREEWALK
};






class CRegVal {
	friend class CRegistry ;

	HANDLE		m_hHeap ;
	LPBYTE		m_pbyteData ;
	DWORD		m_dwDataSize ;
	DWORD		m_dwType ;

	public:
		CRegVal() ;
		_EXPLICIT CRegVal( DWORD ValueToSet );
		_EXPLICIT CRegVal( LPCTSTR ValueToSet );
		CRegVal( const LPBYTE pBuffer, DWORD Size);

#ifdef _REG_MFC_BUILD
		_EXPLICIT CRegVal( const CStringArray & ValueToSet ) ;
#endif // _REG_MFC_BUILD

		~CRegVal() ;

	public:			// functions for the world
		DWORD		GetType() const;
		DWORD		GetSize() const;
		void		SetType( DWORD );

		// direct access to the buffer:
		// NOTE that this buffer is valid as long as this object lives and
		// doesnt change its content !
		// It is much safer to call one of the "GetValue()" methods below !
		LPBYTE			GetRawData() { return m_pbyteData; }
		const LPBYTE	GetRawData() const { return m_pbyteData; }
		
		// the following GetValue()-functions return FALSE, if the type
		// of the registry value doesn't match. In such a case
		// the data (the parameter refers to) stays undefined.
		
		// this function can be used with the following types:
		//		REG_SZ				zero-sized unicode string
		//		REG_EXPAND_SZ		same as REG_SZ (with environ. references)
		//		REG_LINK			symbolic link (unicode)
		//		REG_MULTI_SZ		multiple unicode strings (each separated
		//							by '\0' and a double '\0' at the end)
		// This function is somewhat dangerous, because <strp> will point
		// directly to the data-storage of this class, so if you delete
		// the CRegVal object, the pointer is not longer valid !
		// See the following two functions for alternatives.
		BOOL		GetValue( LPCTSTR & strp ) const ;

#ifdef _REG_MFC_BUILD
		// Use this function for a safer get of a string (except an array)
		// thus:
		//		REG_SZ
		//		REG_EXPAND_SZ
		//		REG_LINK
		BOOL		GetValue( CString & str ) const;

		// Get an array of strings
		// thus:
		//		REG_MULTI_SZ
		BOOL		GetValue( CStringArray & arr ) const;
#endif // _REG_MFC_BUILD

		// this function can be used with the following types:
		//		REG_BINARY			free form binary
		//		REG_RESOURCE_LIST	resource list in the resource map
		//		REG_FULL_RESOURCE_DESCRIPTOR	resource list in the
		//										hardware description
		// if 'size' is not exactly of the same value as the data's size, then
		// this function returns FALSE too.
		BOOL		GetValue( LPBYTE & buffer, DWORD size ) const ;

		// this function can be used with the following types:
		//		REG_DWORD					32-bit number
		//		REG_DWORD_LITTLE_ENDIAN		same as REG_DWORD
		//		REG_DWORD_BIG_ENDIAN		32-bit number
		BOOL		GetValue( DWORD & ) const ;

		// use this function to set a numeric value
		void		SetValue( DWORD Value ) ;
		
		// use the following function to set up a string value. if that string
		// contains references to the environment (such as "%PATH%") then you
		// should set <Expand> to TRUE.
		void		SetValue( LPCTSTR Value, BOOL Expand = FALSE ) ;
		
		// use the following function to set up a multiple string (i.e. string
		// array)
		void		SetValue( LPCTSTR Array [], const DWORD Size ) ;

#ifdef _REG_MFC_BUILD
		void		SetValue( const CStringArray & );
#endif	// _REG_MFC_BUILD
		
		// the following function allows you to set up binary data of any
		// type and size.
		void		SetValue( const LPBYTE Data, const DWORD Size ) ;

		CRegVal &	operator= ( DWORD dwVal ) {
						SetValue( dwVal );
						return * this ;
					}

		CRegVal &	operator= ( LPCTSTR pszVal ) {
						SetValue( pszVal );
						return * this ;
					}

#ifdef _REG_MFC_BUILD
		CRegVal &	operator= ( const CStringArray & arr ) {
						SetValue( arr );
						return * this ;
					}
#endif	// _REG_MFC_BUILD

	private:		// functions for "friend"s only
		BYTE *		AllocateDataBuffer( DWORD ) ;
		void		FreeDataBuffer() ;
} ;


//#endif // !defined(MFC_UTILS_CREGISTRY)
