
#pragma once
//#include <windows.h>
//#include <winuser.h>
#include <afxwin.h>
#include <vector>

#ifdef _DEBUG
	#pragma comment(lib, "Mouse_KeyState_v1.0d.lib")
#else
	#pragma comment(lib, "Mouse_KeyState_v1.0.lib")
#endif	//Endif for Debug mode

//		   Name: CKeyState													1
//
//	Description:
//
//
//		Version 	Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		02.01.00	03/03/98	King		Creation

class CKeyState
{
public:
	static enum
	{
		GLK_F1=128,GLK_F2,GLK_F3,GLK_F4,GLK_F5,GLK_F6,GLK_F7,GLK_F8,GLK_F9,GLK_F10,
		GLK_F11,GLK_F12,GLK_INSERT,GLK_HOME,GLK_PAGE_UP,GLK_DELETE,GLK_END,GLK_PAGE_DOWN,
		GLK_LEFT,GLK_UP,GLK_DOWN,GLK_RIGHT,GLK_ESCAPE,
		GLK_BACK_SPACE,GLK_TAB,GLK_ENTER,GLK_LSHIFT, GLK_RSHIFT,GLK_LCONTROL,GLK_RCONTROL,GLK_LALT, GLK_RALT,
		GLK_LAST
	};
	CKeyState( )
	{

	}

	unsigned char	GetKeyAsChar(void);
	int				GetKeyAsCode(void);

	void			SetKeyAsChar(unsigned char cChar);

	static CPoint	GetMouseScreenPos(void);

	static bool		IsLeftCntlDown();
	static bool		IsRightCntlDown();
	static bool		IsCntlDown();
	
	static bool		IsLeftShiftDown();
	static bool		IsRightShiftDown();
	static bool		IsShiftDown();
	
	static bool		IsRightAltDown();
	static bool		IsLeftAltDown();
	static bool		IsAltDown();

private:
	unsigned char	m_chCharacterValue;
	int				m_nKeyboardCode;
};

//		   Name: CMouseState												2
//
//	Description:
//
//
//		Version 	Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		02.01.00	03/03/98	King		Creation

class CMouseState
{

public:
		static enum EMouseButtonType
		{
			LeftButton, MiddleButton, RightButton, ButtonCount, NoButton
		};

		CMouseState( );

		bool		IsButtonDown(EMouseButtonType eButtonIndex);
	
		CPoint		GetCoords(void);
		void		SetCoords(CPoint& cPoint);
		
		float		GetPercentHeight(void);
		float		GetPercentWidth(void);
		int			GetX(void);
		int			GetY(void);
		
		static CPoint	GetMouseScreenPos(void);

		static bool		IsLeftCntlDown();
		static bool		IsRightCntlDown();
		static bool		IsCntlDown();
		
		static bool		IsLeftShiftDown();
		static bool		IsRightShiftDown();
		static bool		IsShiftDown();
		
		static bool		IsRightAltDown();
		static bool		IsLeftAltDown();
		static bool		IsAltDown();
		
		void		Print(void);
		void		SetButtonDown(EMouseButtonType eButtonIndex, bool bFlag);
		void		SetEventButton(EMouseButtonType eButtonIndex);
		EMouseButtonType	GetEventButton();
		int			operator==(CMouseState& cState);

private:
	bool		m_bButtonDown[ButtonCount];
	CPoint		m_cCoords;
	int			m_nWindowHeight;
	int			m_nWindowWidth;

	EMouseButtonType	m_eEventButton;
};


//		   Name: CKeyMap												
//
//	Description:	
//
//
//		Version 	Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		01.01.00	01/17/00	Weaver		Creation

class CKeyMap  
{
public:
	CKeyMap();
	virtual ~CKeyMap();
	bool	KeyDown(int nCode, unsigned char& cChar);
	void	KeyUp(int nCode, unsigned char& cChar);
	bool IsPressed(unsigned char cChar)	
	{
		for (std::vector<unsigned char>::iterator p = m_vKeys.begin(); p < m_vKeys.end(); p++)
			if (*p == cChar)
				return true;
		return false;
	}
private:
	bool  LCtrlKey()		{ return (GetKeyState(VK_LCONTROL) < 0);	}
	bool  RCtrlKey()		{ return (GetKeyState(VK_RCONTROL) < 0);	}
	bool  CtrlKey()			{ return (LCtrlKey() || LCtrlKey());		}
	
	bool  LShiftKey()		{ return (GetKeyState(VK_LSHIFT) < 0);		}
	bool  RShiftKey()		{ return (GetKeyState(VK_RSHIFT) < 0);		}
	bool  ShiftKey()		{ return (GetKeyState(VK_SHIFT) < 0);		}
	
	bool  RAltKey()			{ return (GetKeyState(VK_RMENU) < 0);		}
	bool  LAltKey()			{ return (GetKeyState(VK_LMENU) < 0);		}
	bool  AltKey()			{ return (GetKeyState(VK_MENU) < 0);		}

	bool CapsLock()			{ return (GetKeyState(VK_CAPITAL) > 0);		}

	void ConvertSpecialKey(int nCode, unsigned char& cKey);
	void ConvertDigit(unsigned char& cChar);
	void ConvertPunctuation(unsigned char& cChar, bool bShift = true);

	void Remove(unsigned char cChar);
	bool m_bRelease;
	std::vector<unsigned char>	m_vKeys;	//Keys down;
};