//		   Name: COpenGLControlBase													
//
//	Description:
//
//
//		Version 	Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		01.01.00	05/21/00	Weaver		Creation
#include "gl\gl.h"  
#include "gl\glu.h" 
#include "gl\glaux.h"
#include "mfcutils_v1.0.h"
#include "mouse_keystate_v1.0.h"
#pragma once

#pragma comment(lib, "opengl32.lib")
#pragma comment(lib, "glu32.lib")
#pragma comment(lib, "glaux.lib")

//		   Name: COpenGLManager 
//
//	Description:	Manage GL Context
//
//
//		Version 	Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		01.01.00	05/21/00	Weaver		Creation


class COpenGLManager  
{
public:
			COpenGLManager();
	virtual ~COpenGLManager();
	void	SetHardwareAccel(bool bFlag);
	bool	IsHardwareAccel()				{ return m_bHardwareAccel; }
	HGLRC	GetContext(HDC hdc);
	HDC		GetDC()							{ return m_hCompatibleDC; }
private:
	HGLRC	m_hOpenGLContext;
	bool	m_bHardwareAccel;
	HDC		m_hCompatibleDC;
};


extern COpenGLManager* g_pManager;
void	KillOpenGLManager();

//		   Name: COpenGLControlBase
//
//	Description:	Baseclass for OpenGL Dialog controls
//
//
//		Version 	Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		01.01.00	05/21/00	Weaver		Creation

class COpenGLControlBase : public CWnd
{
	DECLARE_DYNAMIC(COpenGLControlBase);   
public:
	bool	WindowDepthBufferChanged( );
	void	WindowForceUpdate(bool bOverrideWorkProcSafety = false)					{ Invalidate(); }
	void	WindowGetSize(int& nWidth, int& nHeight)								{ nWidth = m_nWindowWidth; nHeight = m_nWindowHeight; }
	float	WindowGetAspectRatio(void)												{ return GetAspectRatio(); }
	bool	WindowIsAnimating(void)													{ return m_bAnimating; }
	void	WindowStartAnimation(bool bTumbleInduced = false)						{ m_bAnimating = true; }
	void	WindowStopAnimation(bool bOverride = false)								{ m_bAnimating = false; }	

	void	BeginGL()																{ BeginOpenGLDrawing();	} 
	void	EndGL()																	{ EndOpenGLDrawing(); }
	void	CreatePopupMenu( char** pEntries, UINT nID=0);
	void	DoAnimate()																{ vWindowAnimatingOverride();  }
	
	void	PostUpdate();
	void	SetCode(UINT nCode)														{ m_nCode = nCode; }
protected:
	virtual void	vWindowDestroyed( )												{ }
	virtual void	vWindowMouseButtonDoublePress(CMouseState cMouse)				{ }
	virtual void	vWindowMouseButtonRelease(CMouseState cMouse)					{ }	
	virtual void	vWindowMouseWheelMotion(CMouseState cMouse, short zDelta)		{ }
	virtual void	vWindowExposure( )												{ }			
	virtual	void	vWindowPostSwap(void)											{ }
	virtual	void	vWindowDraw( )													{ }								
	virtual void	vWindowInitialExposure( )										{ }
	virtual void	vWindowMouseButtonPress(CMouseState cMouse)						{ }
	virtual void	vWindowResized(int nX, int nY)									{ }
	virtual void	vWindowKeyboardHandler(CKeyState cKey)							{ }
	virtual void	vWindowAnimatingOverride( )										{ WindowForceUpdate(); }
	virtual void	vWindowMouseMotion(CMouseState cMouse, CPoint cDelta)			{ }
	virtual void	vWindowMenuItemSelected( UINT nIndex)							{ }
	
private:
	static BOOL RegisterOpenGLWindowClass();
	void		Draw();									

private:	//private member variables		
	bool		m_bMousePressedInThisWindow;
	
protected:	//attributes
	//OpenGL Varialbles
	HGLRC				m_hRC; 
    HDC					m_hDC;  
	CDC*				m_pDC;
	float				m_fLastDepth;
	//Flags
	bool				m_bAnimating;
	bool				m_bMoving;
	bool				m_bInitialized;
	
	UINT				m_nCode;
	CWinUMenu			m_cMenu;
	CMouseState			m_cCurrentMouse;
	int					m_nWindowHeight;
	int					m_nWindowWidth;
	CKeyMap				m_cKeyMap;

public:		//Functions
	COpenGLControlBase(); 
   ~COpenGLControlBase();
	bool		IsAnimating();

	void		Animate( bool bFlag=true)											{ m_bAnimating = bFlag;	 	}
	int			GetWidth()															{ return m_nWindowWidth;	}
	int			GetHeight()															{ return m_nWindowHeight;	}
	float		GetAspectRatio()													{ return (float( m_nWindowWidth) / float( m_nWindowHeight) ); }	
    void		BeginOpenGLDrawing();
    void		EndOpenGLDrawing();
	
protected:
    virtual void InitializeOpenGLWindow();
    virtual void UninitializeOpenGLWindow();
    virtual void SetupPixelFormat(HDC hDC);
    virtual void CustomizePixelFormat(PIXELFORMATDESCRIPTOR * pfd){}



	//MFC Functions
protected:
    virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
    virtual void PreSubclassWindow();
    afx_msg void OnDestroy();
    afx_msg void OnPaint();
    afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnRButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnRButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnMButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnMButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnSysKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnSysKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg BOOL OnMouseWheel(UINT nFlags, short zDelta, CPoint pt);
	afx_msg void OnNcPaint( );
	afx_msg void OnMenu( UINT );
	afx_msg void OnTimer( UINT nIDEvent );
    DECLARE_MESSAGE_MAP()
};
