// CUserCar Class Implementation
// 
// Description : inherits from CCar, adds user interface
//

// includes
#include "stdafx.h"
#include "UserCar.h"
#include "Mouse_KeyState_v1.0.h"

///////////////////////////////////////////////////////////////////
// CUserCar::CUserCar()
// Description:	constructor
///////////////////////////////////////////////////////////////////
CUserCar::CUserCar()
{
	m_bInsideCar = false;
	InitCamera();
	m_fMaxSpeed = 6.25f;
	m_bInGame = false;
}

///////////////////////////////////////////////////////////////////
// CUserCar::Update(bool bUp, bool bDown, bool bLeft, bool bRight, int* pEnv)
// Description:	gets key input and drives car accordingly
///////////////////////////////////////////////////////////////////
bool CUserCar::Update(bool bUp, bool bDown, bool bLeft, bool bRight, int* pEnv)
{
	CCar::Update();
	m_bIsTurning = false;

	// speed
	if (bUp)
		SpeedUp();
	if (bDown)
		Brake();
	if(!bUp && !bDown)
		Coast();

	// turning
	if(GetSpeed() > 0.0f)
	{
		if (bLeft)
			TurnLeft();
		if (bRight)
			TurnRight();
	}
	else // reverse turning
	{
		if (bLeft)
			TurnRight();
		if (bRight)
			TurnLeft();
	}
	
	// return true to play sound if hitting something
	bool bCrash = Move(pEnv);

	// set camera for inside or outside of car
	if(m_bInsideCar)
	{	// inside
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		gluLookAt(GetPos().data[0], GetPos().data[1], GetPos().data[2] + 5.0f, 
				GetPos().data[0] + GetDir().data[0], GetPos().data[1] + GetDir().data[1], GetPos().data[2] + GetDir().data[2] + 5.0f, 
				0.0, 0.0, 1.0);  // set to track normal when we do 3D tracks
	}
	else
	{	// outside
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		gluLookAt(GetPos().data[0] - 25*m_cOldDirection[0].data[0], GetPos().data[1] - 25*m_cOldDirection[0].data[1], GetPos().data[2] + 8.0f, 
				GetPos().data[0], GetPos().data[1], GetPos().data[2] + 7.0f, 
				0.0, 0.0, 1.0);  // set to track normal when we do 3D tracks
	}

	// store old directions in array, lets car turn a little faster than the camera
	for(int i = 0; i < 14; i++)
		m_cOldDirection[i] = m_cOldDirection[i+1];
	m_cOldDirection[14] = GetDir();

	return bCrash;
}

///////////////////////////////////////////////////////////////////
// CUserCar::Draw()
// Description:	draw the user car
///////////////////////////////////////////////////////////////////
void CUserCar::Draw()
{		
	if(!m_bInsideCar)
	{
		glPushMatrix();
			glTranslatef(GetPos().data[0], GetPos().data[1], GetPos().data[2]);
			if(GetDir().data[1] > 0.0f)
				glRotatef(57.29578f * GUAngleBetween(GetDir(), CPnt(-1.0f, 0.0f, 0.0f)), 0.0f, 0.0f, -1.0f);
			else
				glRotatef(57.29578f * GUAngleBetween(GetDir(), CPnt(-1.0f, 0.0f, 0.0f)), 0.0f, 0.0f, 1.0f);
			CCar::Draw();
		glPopMatrix();
	}
	DrawInfo();
}

///////////////////////////////////////////////////////////////////
// CUserCar::DrawInfo()
// Description:	draw the info bar and laps
///////////////////////////////////////////////////////////////////
void CUserCar::DrawInfo()
{
	
	// change viewport so it stays still (to the observer)
	glViewport (0, 0, 640, 480);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(50.0f, 1.333f, 1.0f, 100.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(0.0f, 0.0f, -5.0f, 0.0f, 0.0f, 5.0f, 0.0f, 1.0f, 0.0f);
	glDisable(GL_LIGHTING);

	if(m_bInGame)
	{
		// draw translucent background
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glColor4f(0.0f, 0.0f, 0.2f, 0.5f);
		glBegin(GL_POLYGON);
			glVertex3f(-3.5f, -3.0f, 0.0f);
			glVertex3f(-3.5f, -2.1f, 0.0f);
			glVertex3f(3.5f, -2.1f, 0.0f);
			glVertex3f(3.5f, -3.0f, 0.0f);
		glEnd();
		glDisable(GL_BLEND);
		glLineWidth(3.0f);
		glColor4f(0.0f, 0.0f, 0.0f, 1.0f);
		glBegin(GL_LINES);
			glVertex3f(-3.5f, -2.1f, 0.0f);
			glVertex3f(3.5f, -2.1f, 0.0f);
		glEnd();

		// draw text
		glRotatef(180.0f, 0.0f, 1.0f, 0.0f);
		glScalef(0.2f, 0.2f, 0.2f);
		glPushAttrib(GL_POLYGON_BIT | GL_ENABLE_BIT);
			glTranslatef(-15.25f, -11.25f, 0.2f);
			glPushMatrix();
				glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
				glListBase(m_dlText);
				glCallLists(5, GL_UNSIGNED_BYTE, "MPH: ");

				char strTemp[10];
				sprintf(strTemp, "%d", (int)(GetVel().magnitude() * 30.0f));
				int nTemp;
				if((int)(GetVel().magnitude() * 30.0f) > 99)
					nTemp = 3;
				else if((int)(GetVel().magnitude() * 30.0f) > 9)
					nTemp = 2;
				else
					nTemp = 1;
				if(GetSpeed() < 0.0f)
				{
					nTemp = 1;
					strTemp[0] = '-';
				}
				glListBase(m_dlText);
				glCallLists(nTemp, GL_UNSIGNED_BYTE, strTemp);
			glPopMatrix();
			glTranslatef(14.0f, 0.0f, 0.0f);
			glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
			glListBase(m_dlText);
			glCallLists(8, GL_UNSIGNED_BYTE, GetTime().c_str());

			glTranslatef(9.0f, 0.0f, 0.0f);
			glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
			glListBase(m_dlText);
			glCallLists(6, GL_UNSIGNED_BYTE, "Place:");
			glCallLists(2, GL_UNSIGNED_BYTE, GetPlaceString().c_str());
		glPopAttrib();

		// draw laps
		glPushAttrib(GL_POLYGON_BIT | GL_ENABLE_BIT);
			glPushMatrix();
				glTranslatef(-5.0f, 21.0f, 0.0f);
				glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
				glListBase(m_dlText);
				glCallLists(4, GL_UNSIGNED_BYTE, "Lap ");
				glTranslatef(0.0f, 0.2f, 0.0f);
				glCallLists(2, GL_UNSIGNED_BYTE, GetLapString().c_str());
				glTranslatef(0.0f, -0.2f, 0.0f);
				glCallLists(1, GL_UNSIGNED_BYTE, "/");
				glTranslatef(0.0f, -0.2f, 0.0f);
				glCallLists(2, GL_UNSIGNED_BYTE, GetNumLapsString().c_str());
			glPopMatrix();
		glPopAttrib();	


		if(((float)(clock() - m_fStartWords) / CLOCKS_PER_SEC) < 6.0f)
		{
			glPushAttrib(GL_POLYGON_BIT | GL_ENABLE_BIT);
				glPushMatrix();
						glScalef(4.0f,4.0f,4.0f);
						glTranslatef(-4.45f, 2.85f, 0.0f);
						glColor4f(0.0f, 1.0f, 0.0f, 1.0f);
					glListBase(m_dlText);
					glCallLists(3, GL_UNSIGNED_BYTE, "Go!");
				glPopMatrix();
			glPopAttrib();	
		}

		// draw the finished stuff
		if(GetFinished())
		{
			if(WonRace())
			{
				glPushAttrib(GL_POLYGON_BIT | GL_ENABLE_BIT);
					glPushMatrix();
						glScalef(4.0f,4.0f,4.0f);
						glTranslatef(-5.25f, 4.0f, 0.0f);
						glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
						glListBase(m_dlText);
						glCallLists(7, GL_UNSIGNED_BYTE, "Winner!");
					glPopMatrix();
				glPopAttrib();	
			}
			else
			{
				glPushAttrib(GL_POLYGON_BIT | GL_ENABLE_BIT);
					glPushMatrix();
						glScalef(4.0f,4.0f,4.0f);
						glTranslatef(-5.0f, 4.0f, 0.0f);
						glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
						glListBase(m_dlText);
						glCallLists(6, GL_UNSIGNED_BYTE, "Loser!");
					glPopMatrix();
				glPopAttrib();	
			}
		}
	}
	else
	{
		glRotatef(180.0f, 0.0f, 1.0f, 0.0f);
		if(((float)(clock() - m_fStartWords) / CLOCKS_PER_SEC) < 2.0f)
		{
			glPushAttrib(GL_POLYGON_BIT | GL_ENABLE_BIT);
				glPushMatrix();
					glTranslatef(-1.25f, 0.0f, 0.0f);
					glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
					glListBase(m_dlText);
					glCallLists(6, GL_UNSIGNED_BYTE, "Ready!");
				glPopMatrix();
			glPopAttrib();	
		}
		else if(((float)(clock() - m_fStartWords) / CLOCKS_PER_SEC) < 6.5f)
		{
			glPushAttrib(GL_POLYGON_BIT | GL_ENABLE_BIT);
				glPushMatrix();
					glTranslatef(-0.75f, 0.0f, 0.0f);
					glColor4f(1.0f, 1.0f, 0.0f, 1.0f);
					glListBase(m_dlText);
					glCallLists(4, GL_UNSIGNED_BYTE, "Set!");
				glPopMatrix();
			glPopAttrib();	
		}
	}


	// set viewport back
	glEnable(GL_LIGHTING);
	glViewport (0, 0, 640, 480);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(57.0f, 1.333f, 1.0f, 3000.0f);
	if(m_bInsideCar)
	{
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		gluLookAt(GetPos().data[0], GetPos().data[1], GetPos().data[2] + 5.0f, 
				GetPos().data[0] + GetDir().data[0], GetPos().data[1] + GetDir().data[1], GetPos().data[2] + GetDir().data[2] + 5.0f, 
				0.0, 0.0, 1.0);  // set to track normal when we do 3D tracks
	}
	else
	{
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		gluLookAt(GetPos().data[0] - 25*m_cOldDirection[0].data[0], GetPos().data[1] - 25*m_cOldDirection[0].data[1], GetPos().data[2] + 8.0f, 
				GetPos().data[0], GetPos().data[1], GetPos().data[2] + 7.0f, 
				0.0, 0.0, 1.0);  // set to track normal when we do 3D tracks
	}
}

///////////////////////////////////////////////////////////////////
// CUserCar::GetTime()
// Description:	function to get time for high score screen
///////////////////////////////////////////////////////////////////
string CUserCar::GetTime()
{
	char strMilli[20];
	char strSec[20];
	char strMin[20];
	string str4;
	string str5;
	string str6;

	float fCurrentTime = (float)clock();
	float duration = (float)(fCurrentTime - GetStartTime()) / CLOCKS_PER_SEC;

   	int minutes = (int)duration/60;
	if(minutes > 0)
		duration = duration - (minutes*60);
	int seconds = (int)duration;
	sprintf(strMilli, "%.2f", duration);
	sprintf(strSec, "%.2d", seconds);
	sprintf(strMin, "%.2d", minutes);

	str4 = strMilli;
	str5 = strSec;
	str6 = strMin;

	if(minutes > 0)
		str6 = str6 + ":" + str5;
	else
		str6 = "00:" + str5;
	str6+=str4.c_str()[str4.length()-3];
	str6+=str4.c_str()[str4.length()-2];
	str6+=str4.c_str()[str4.length()-1];
	
	return str6;
}

///////////////////////////////////////////////////////////////////
// CUserCar::InitCamera()
// Description:	clear out the array of directions to reset the game view
///////////////////////////////////////////////////////////////////
void CUserCar::InitCamera()
{
	for(int i = 0; i < 15; i++)
		m_cOldDirection[i].set(-1.0f, 0.0f, 0.0f);
}
